@@ -1,7 +1,7 @@
 /* Help strings.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2001 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2004 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -20,7 +20,6 @@
 	Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 	02111-1307, USA.  */
 
-
 const char * const ABOUT_HELP[5] = {
   "Syntax: About",
   "Abbreviation: About",
@@ -50,7 +49,7 @@ const char * const ADJUSTVIEW_HELP[20] = {
   "limits of the current TAB size).",
 };
 
-const char * const AUTOINDENT_HELP[43] = {
+const char * const AUTOINDENT_HELP[22] = {
   "Syntax: AutoIndent [0|1]",
   "Abbreviation: AI",
   "",
@@ -71,28 +70,7 @@ const char * const AUTOINDENT_HELP[43] = {
   "are not satisfied with the indentation, just give the `Undo' command and",
   "the indentation will disappear (but the new line will remain in place,",
   "since its creation has been recorded as a separate action). See",
-  "Undo",
-  "",
-  "This is ne.info, produced by makeinfo version 4.0 from ne.texinfo.",
-  "",
-  "   This file documents ne 1.19, a text editor for UN*X.",
-  "",
-  "   Copyright (C) 1993-1998 Sebastiano Vigna.  Copyright (C) 1999-2000",
-  "Todd M. Lewis and Sebastiano Vigna.",
-  "",
-  "   Permission is granted to make and distribute verbatim copies of this",
-  "manual provided the copyright notice and this permission notice are",
-  "preserved on all copies.",
-  "",
-  "   Permission is granted to copy and distribute modified versions of",
-  "this manual under the conditions for verbatim copying, provided that",
-  "the entire resulting derived work is distributed under the terms of a",
-  "permission notice identical to this one.",
-  "",
-  "   Permission is granted to copy and distribute translations of this",
-  "manual into another language, under the above conditions for modified",
-  "versions, except that this permission notice may be stated in a",
-  "translation approved by the Free Software Foundation.",
+  "Undo.",
 };
 
 const char * const AUTOPREFS_HELP[14] = {
@@ -156,7 +134,7 @@ const char * const CAPITALIZE_HELP[6] = {
   "Abbreviation: CA",
   "",
   "acts exactly like `ToUpper', but capitalizes, that is, makes the first",
-  "letter upper case and the other ones lower case. See ToUpper",
+  "letter upper case and the other ones lower case. See ToUpper.",
 };
 
 const char * const CASESEARCH_HELP[12] = {
@@ -180,7 +158,7 @@ const char * const CENTER_HELP[8] = {
   "centers N lines from the cursor position onwards. If N is not",
   "specified, it is assumed to be one. The lines are centered with spaces,",
   "relatively to the value of the right margin as set by the `RightMargin'",
-  "command. See RightMargin",
+  "command. See RightMargin.",
 };
 
 const char * const CLEAR_HELP[7] = {
@@ -281,7 +259,7 @@ const char * const DELETELINE_HELP[9] = {
   "",
   "deletes N lines starting from the current cursor position (putting the",
   "last one in the temporary buffer, from which it can be undeleted. See",
-  "UndelLine) If the optional N argument is not specified, it is",
+  "UndelLine.) If the optional N argument is not specified, it is",
   "assumed to be one. Note that this action is in no way inverse with",
   "respect to `InsertLine'.",
 };
@@ -322,10 +300,10 @@ const char * const ESCAPE_HELP[7] = {
   "",
   "toggles the menus on and off, or escapes from the input line. This",
   "command is mainly useful for reprogramming the menu activator, and it is",
-  "never registered while recording a macro. See Record",
+  "never registered while recording a macro. See Record.",
 };
 
-const char * const ESCAPETIME_HELP[14] = {
+const char * const ESCAPETIME_HELP[16] = {
   "Syntax: EscapeTime [N]",
   "Abbreviation: ET",
   "",
@@ -334,7 +312,9 @@ const char * const ESCAPETIME_HELP[14] = {
   "default value of 10 is too low: in this case, escape sequences (e.g.,",
   "those of the arrow keys) could be erroneously broken into an escape and",
   "some spurious characters.  Rising the escape time usually solves this",
-  "problem. Allowed values range from 0 to 255.",
+  "problem. Allowed values range from 0 to 255. Note that you can",
+  "accelerate the recognition of the <Escape> key by hitting it twice in a",
+  "row.",
   "",
   "   Note that the escape time is global to `ne', and it is not saved.",
   "However, you can add an `EscapeTime' command manually to a preferences",
@@ -353,7 +333,7 @@ const char * const EXEC_HELP[13] = {
   "manually programmed macros.",
   "",
   "   Note that if the command you specify does not appear in `ne''s",
-  "internal tables, it is considered to be a macro name. See Macro",
+  "internal tables, it is considered to be a macro name. See Macro.",
 };
 
 const char * const EXIT_HELP[6] = {
@@ -389,7 +369,7 @@ const char * const FIND_HELP[12] = {
   "occurrence of the pattern, or an error message is given. The direction",
   "and the case sensitivity of the search are established by the value of",
   "the back search and case sensitive search flags. See See SearchBack",
-  "and See CaseSearch",
+  "and See CaseSearch.",
   "",
   "   If the optional argument PATTERN is not specified, you can enter it",
   "on the input line, the default being the last pattern used.",
@@ -403,13 +383,13 @@ const char * const FINDREGEXP_HELP[12] = {
   ". The cursor is positioned on the first string matching the expression.",
   "The direction and the kind of search are established by the value of",
   "the back search and case sensitive search flags. See See SearchBack",
-  "and See CaseSearch",
+  "and See CaseSearch.",
   "",
   "   If the optional argument PATTERN is not specified, you can enter it",
   "on the input line, the default being the last pattern used.",
 };
 
-const char * const FLAGS_HELP[25] = {
+const char * const FLAGS_HELP[27] = {
   "Syntax: Flags",
   "Abbreviation: FLAG",
   "",
@@ -417,23 +397,25 @@ const char * const FLAGS_HELP[25] = {
   "commands. It is not recorded when recording a macro.",
   "",
   "     FLAG COMMAND      ABBR  DESCRIPTION",
-  "      i   Insert        I    inserts new characters (vs. replacing)",
-  "      a   AutoIndent    AI   aligns cursor under previous line after <Return>",
-  "      b   BackSearch    BS   searches search backward rather than forward",
-  "      c   CaseSearch    CS   searches are case sensitive",
-  "      w   WordWrap      WW   breaks long lines as you type",
-  "      f   FreeForm      FF   allows cursor to move beyond the end of lines",
-  "      p   AutoPrefs     AP   use automatic preferences based on file extension",
-  "      v   VerboseMacros VM   record macros using use long command names",
-  "      u   DoUndo        DU   record edits for later undoing",
-  "      r   ReadOnly      RO   changes are not allowed",
-  "      B   Binary        B    affects file loading/saving",
-  "      M   Mark          M    mark set for line-oriented block operations",
-  "      V   MarkVert      MV   like mark, but block is rectangle",
-  "      R   Record        REC  actions are being recorded in a macro",
-  "      P   PreserveCR    PCR  affects how <CR> chars are loaded from files",
-  "      C   CRLF          CRLF use CR/LF as line terminator",
-  "      *   Modified      MOD  document has been modified since last saved",
+  "       i   Insert        I    inserts new characters (vs. replacing)",
+  "       a   AutoIndent    AI   aligns cursor under previous line after <Return>",
+  "       b   BackSearch    BS   searches search backward rather than forward",
+  "       c   CaseSearch    CS   searches are case sensitive",
+  "       w   WordWrap      WW   breaks long lines as you type",
+  "       f   FreeForm      FF   allows cursor to move beyond the end of lines",
+  "       p   AutoPrefs     AP   use automatic preferences based on file extension",
+  "       v   VerboseMacros VM   record macros using use long command names",
+  "       u   DoUndo        DU   record edits for later undoing",
+  "       r   ReadOnly      RO   changes are not allowed",
+  "       B   Binary        B    affects file loading/saving",
+  "       M   Mark          M    mark set for line-oriented block operations",
+  "       V   MarkVert      MV   like mark, but block is rectangle",
+  "       R   Record        REC  actions are being recorded in a macro",
+  "       P   PreserveCR    PCR  affects how <CR> chars are loaded from files",
+  "       C   CRLF          CRLF use CR/LF as line terminator",
+  "       *   Modified      MOD  document has been modified since last saved",
+  "       @   UTF8IO        U8IO I/O (keyboard and terminal) are UTF-8 encoded",
+  "     A/8/U UTF8          U8   the document encoding (ASCII, 8-bit or UTF-8)",
 };
 
 const char * const FLASH_HELP[6] = {
@@ -441,7 +423,7 @@ const char * const FLASH_HELP[6] = {
   "Abbreviation: FL",
   "",
   "acts as `Beep', but interchanging the words \"beep\" and \"flash\".  Same",
-  "comments apply. See Beep",
+  "comments apply. See Beep.",
 };
 
 const char * const FREEFORM_HELP[18] = {
@@ -499,11 +481,11 @@ const char * const GOTOMARK_HELP[9] = {
   "Syntax: GotoMark",
   "Abbreviation: GM",
   "",
-  "moves the cursor to the current mark, if it exists. See Mark",
+  "moves the cursor to the current mark, if it exists. See Mark.",
   "",
   "   `GotoMark' is mainly useful if you forgot where you started marking.",
   "If you want to record a position in a file and jump to it later, you",
-  "may want to use a bookmark. See SetBookmark",
+  "may want to use a bookmark. See SetBookmark.",
 };
 
 const char * const HELP_HELP[15] = {
@@ -520,7 +502,7 @@ const char * const HELP_HELP[15] = {
   "",
   "   Invocations of the `Help' command are never registered while",
   "recording macros so that you can safely access the help system while",
-  "recording. See Record",
+  "recording. See Record.",
 };
 
 const char * const HEXCODE_HELP[7] = {
@@ -561,11 +543,11 @@ const char * const INSERTCHAR_HELP[20] = {
   "   Note that inserting a line feed (10) is completely different from",
   "inserting a line with `InsertLine'. `InsertChar 10' puts the control",
   "char `<Control>-J' in the text at the current cursor position.  See",
-  "InsertLine",
+  "InsertLine.",
   "",
   "   Note also that `SaveMacro' converts `InsertChar' commands into a",
   "possibly smaller number of `InsertString' commands.  (See",
-  "SaveMacro) This makes macros easier to read and edit.",
+  "SaveMacro.) This makes macros easier to read and edit.",
 };
 
 const char * const INSERTLINE_HELP[7] = {
@@ -587,7 +569,7 @@ const char * const INSERTSTRING_HELP[12] = {
   "indent, et cetera) are applied.",
   "",
   "   Note that `SaveMacro' converts `InsertChar' commands into a possibly",
-  "smaller number of `InsertString' commands.  (See SaveMacro) This",
+  "smaller number of `InsertString' commands.  (See SaveMacro.) This",
   "makes macros easier to read and edit.",
 };
 
@@ -613,7 +595,7 @@ const char * const LOADAUTOPREFS_HELP[7] = {
   "",
   "loads the preferences file in `~/.ne' associated with the current",
   "document's file name extension. If the current file name has no",
-  "extension, the default preferences are loaded.  See AutoPrefs",
+  "extension, the default preferences are loaded.  See AutoPrefs.",
 };
 
 const char * const LOADPREFS_HELP[16] = {
@@ -626,7 +608,7 @@ const char * const LOADPREFS_HELP[16] = {
   "   If the optional FILENAME argument is not specified, the file",
   "requester is opened, and you are prompted to select a file. (You can",
   "inhibit the file requester opening by using the `NoFileReq' command;",
-  "see NoFileReq) If you escape from the file requester, you can",
+  "see NoFileReq.) If you escape from the file requester, you can",
   "input the file name on the command line.",
   "",
   "   Note that a preferences file is just a macro containing only option",
@@ -643,7 +625,7 @@ const char * const MACRO_HELP[31] = {
   "   If the optional FILENAME argument is not specified, the file",
   "requester is opened, and you are prompted to select a file. (You can",
   "inhibit the file requester opening by using the `NoFileReq' command;",
-  "see NoFileReq)",
+  "see NoFileReq.)",
   "",
   "   If you escape from the file requester, you can input the file name",
   "on the command line.",
@@ -878,7 +860,7 @@ const char * const OPEN_HELP[18] = {
   "   If the optional FILENAME argument is not specified, the file",
   "requester is opened, and you are prompted to select a file. (You can",
   "inhibit the file requester opening by using the `NoFileReq' command;",
-  "see NoFileReq)",
+  "see NoFileReq.)",
   "",
   "   If you escape from the file requester, you can input the file name",
   "on the command line, the default being the current document name, if",
@@ -898,7 +880,7 @@ const char * const OPENCLIP_HELP[14] = {
   "   If the optional FILENAME argument is not specified, the file",
   "requester will open and you will be prompted to select a file. (You can",
   "inhibit the file requester opening by using the `NoFileReq' command;",
-  "see NoFileReq)",
+  "see NoFileReq.)",
   "",
   "   If you escape from the file requester, you can enter the file name",
   "on the input line.",
@@ -914,7 +896,7 @@ const char * const OPENMACRO_HELP[14] = {
   "   If the optional FILENAME argument is not specified, the file",
   "requester is opened, and you are prompted to select a file. (You can",
   "inhibit the file requester opening by using the `NoFileReq' command;",
-  "see NoFileReq)",
+  "see NoFileReq.)",
   "",
   "   If you escape from the file requester, you can input the file name",
   "on the command line.",
@@ -925,7 +907,7 @@ const char * const OPENNEW_HELP[6] = {
   "Abbreviation: ON",
   "",
   "is the same as `Open', but loads the file specified by the FILENAME",
-  "string into a new document. See Open",
+  "string into a new document. See Open.",
 };
 
 const char * const PAGEDOWN_HELP[6] = {
@@ -951,7 +933,7 @@ const char * const PARAGRAPH_HELP[24] = {
   "reformats N paragraphs from the cursor position onwards. If N is not",
   "specified, it is assumed to be one. The paragraph are formatted",
   "relatively to the value of the right margin as set by the `RightMargin'",
-  "command. See RightMargin",
+  "command. See RightMargin.",
   "",
   "   `ne''s notion of a paragraph includes the current non-blank line",
   "(regardless of its leading white space) and all subsequent non-blank",
@@ -1011,7 +993,7 @@ const char * const POPPREFS_HELP[24] = {
   "",
   "pops N sets of preferences from the prefs stack (where they were placed",
   "previously by `PushPrefs') and applies those preferences to the current",
-  "buffer. See PushPrefs If not specified, N defaults to one. Note",
+  "buffer. See PushPrefs. If not specified, N defaults to one. Note",
   "that the preferences stack is global, not buffer specific. Therefore",
   "you could `PushPrefs' one buffer's preferences, switch buffers, then",
   "`PopPrefs' those settings altering the preferences for the second",
@@ -1081,7 +1063,7 @@ const char * const PUSHPREFS_HELP[24] = {
   "",
   "pushes N copies of the user preferences onto a stack. If not specified,",
   "N defaults to one. Use the `PopPrefs' command to pop preferences off",
-  "the stack and restore the values. See PopPrefs Note that the",
+  "the stack and restore the values. See PopPrefs. Note that the",
   "preferences stack is global, not buffer-specific, so you could",
   "`PushPrefs' one buffer's preferences, switch buffers, then `PopPrefs'",
   "those preferences, thereby altering the preferences for the second",
@@ -1116,7 +1098,7 @@ const char * const READONLY_HELP[13] = {
   "sets the read only flag. When this flag is true, no editing can be",
   "performed on the document (any such attempt produces an error message).",
   "This flag is automatically set whenever you open a file that you cannot",
-  "write to.  See Open",
+  "write to.  See Open.",
   "",
   "   If you invoke `ReadOnly' with no arguments, it will toggle the flag.",
   "If you specify 0 or 1, the flag will be set to false or true,",
@@ -1154,7 +1136,7 @@ const char * const REDO_HELP[8] = {
   "redoes the last N actions undone by `Undo' (as long as you don't take",
   "any actions that change the text between the `Undo' and `Redo'",
   "commands). If N is not specified, it is assumed to be one. You can only",
-  "`Redo' actions that have been `Undo'ne. See Undo",
+  "`Redo' actions that have been `Undo'ne. See Undo.",
 };
 
 const char * const REFRESH_HELP[11] = {
@@ -1221,7 +1203,7 @@ const char * const REPLACE_HELP[46] = {
   "",
   "   * reversing the search direction (`Backward' or `Forward'); this",
   "     choice will also modify the value of the back search flag.  See",
-  "     SearchBack",
+  "     SearchBack.",
   "",
   "",
   "   `Replace' is mainly useful for interactive editing. `ReplaceOnce',",
@@ -1243,7 +1225,7 @@ const char * const REPLACE_HELP[46] = {
   "process. When the macro is played, you will again have to choose which",
   "actions to perform. If you want to apply automatic replacement of",
   "strings for a certain number of times, you should look at See",
-  "ReplaceOnce, See ReplaceAll and See RepeatLast",
+  "ReplaceOnce, See ReplaceAll and See RepeatLast.",
 };
 
 const char * const REPLACEALL_HELP[12] = {
@@ -1257,7 +1239,7 @@ const char * const REPLACEALL_HELP[12] = {
   "the input line, the default being the last string used.",
   "",
   "   Note that `Undo' will restore _all_ the occurrences of the search",
-  "pattern replaced by `ReplaceAll'. See Undo",
+  "pattern replaced by `ReplaceAll'. See Undo.",
 };
 
 const char * const REPLACEONCE_HELP[10] = {
@@ -1294,7 +1276,7 @@ const char * const SAVE_HELP[12] = {
   "",
   "   If the current document is unnamed, the file requester will open and",
   "you will be prompted to select a file. (You can inhibit the file",
-  "requester opening by using the `NoFileReq' command; see NoFileReq)",
+  "requester opening by using the `NoFileReq' command; see NoFileReq.)",
   "",
   "   If you escape from the file requester, you can input the file name",
   "on the command line.",
@@ -1309,7 +1291,7 @@ const char * const SAVEAS_HELP[14] = {
   "   If the optional FILENAME argument is not specified, the file",
   "requester will open and you will be prompted to select a file. (You can",
   "inhibit the file requester opening by using the `NoFileReq' command;",
-  "see NoFileReq)",
+  "see NoFileReq.)",
   "",
   "   If you escape from the file requester, you can enter the file name",
   "on the input line, the default being the current document name, if",
@@ -1322,7 +1304,7 @@ const char * const SAVEAUTOPREFS_HELP[7] = {
   "",
   "saves the current preferences on the file in `~/.ne' associated with",
   "the current document's file name extension. If the current file name",
-  "has no extension, an error message is issued.  See AutoPrefs",
+  "has no extension, an error message is issued.  See AutoPrefs.",
 };
 
 const char * const SAVECLIP_HELP[13] = {
@@ -1334,7 +1316,7 @@ const char * const SAVECLIP_HELP[13] = {
   "   If the optional FILENAME argument is not specified, the file",
   "requester will open and you will be prompted to select a file. (You can",
   "inhibit the file requester opening by using the `NoFileReq' command;",
-  "see NoFileReq)",
+  "see NoFileReq.)",
   "",
   "   If you escape from the file requester, you can enter the file name",
   "on the input line.",
@@ -1357,7 +1339,7 @@ const char * const SAVEMACRO_HELP[20] = {
   "   If the optional FILENAME argument is not specified, the file",
   "requester is opened, and you are prompted to select a file. (You can",
   "inhibit the file requester opening by using the `NoFileReq' command;",
-  "see NoFileReq)",
+  "see NoFileReq.)",
   "",
   "   If you escape from the file requester, you can input the file name",
   "on the command line.",
@@ -1366,7 +1348,7 @@ const char * const SAVEMACRO_HELP[20] = {
   "recorded. The macros can then be loaded as normal text files for",
   "further editing, if necessary. Note that `SaveMacro' converts",
   "`InsertChar' commands into a possibly smaller number of `InsertString'",
-  "commands. (See InsertChar, and See InsertString)  This makes",
+  "commands. (See InsertChar, and See InsertString.)  This makes",
   "macros easier to read and edit.",
 };
 
@@ -1379,7 +1361,7 @@ const char * const SAVEPREFS_HELP[11] = {
   "   If the optional FILENAME argument is not specified, the file",
   "requester is opened, and you are prompted to select a file. (You can",
   "inhibit the file requester opening by using the `NoFileReq' command;",
-  "see NoFileReq) If you escape from the file requester, you can",
+  "see NoFileReq.) If you escape from the file requester, you can",
   "input the file name on the command line.",
 };
 
@@ -1396,7 +1378,7 @@ const char * const SEARCHBACK_HELP[14] = {
   "flag is true.",
   "",
   "   Note that this flag also can be set through interactions with the",
-  "`Replace' command. See Replace",
+  "`Replace' command. See Replace.",
 };
 
 const char * const SELECTDOC_HELP[14] = {
@@ -1412,7 +1394,7 @@ const char * const SELECTDOC_HELP[14] = {
   "",
   "   `SelectDoc' is especially useful if you have a large number of",
   "documents open (say, more than 10). Otherwise, `NextDoc' and `PrevDoc'",
-  "should be enough. See NextDoc, and See PrevDoc",
+  "should be enough. See NextDoc, and See PrevDoc.",
 };
 
 const char * const SETBOOKMARK_HELP[7] = {
@@ -1502,7 +1484,7 @@ const char * const TOGGLESEOF_HELP[10] = {
   "",
   "   This kind of toggling command (also see ToggleSEOL) is very",
   "useful in order to gain some keystrokes on systems with very few keys.",
-  "See also See MoveSOF, and See MoveEOF",
+  "See also See MoveSOF, and See MoveEOF.",
 };
 
 const char * const TOGGLESEOL_HELP[10] = {
@@ -1514,14 +1496,14 @@ const char * const TOGGLESEOL_HELP[10] = {
   "",
   "   This kind of toggling command (also see ToggleSEOF) is very",
   "useful in order to gain some keystrokes on systems with very few keys.",
-  "See also See MoveSOL, and See MoveEOL",
+  "See also See MoveSOL, and See MoveEOL.",
 };
 
 const char * const TOLOWER_HELP[5] = {
   "Syntax: ToLower [N]",
   "Abbreviation: TL",
   "",
-  "acts exactly like `ToUpper', but lowers the case. See ToUpper",
+  "acts exactly like `ToUpper', but lowers the case. See ToUpper.",
 };
 
 const char * const TOUPPER_HELP[15] = {
@@ -1586,7 +1568,7 @@ const char * const UNDELLINE_HELP[15] = {
   "with clips.",
   "",
   "   Note that `UndelLine' works independently of the status of the undo",
-  "flag. See DoUndo",
+  "flag. See DoUndo.",
 };
 
 const char * const UNDO_HELP[9] = {
@@ -1597,7 +1579,7 @@ const char * const UNDO_HELP[9] = {
   "one. After you undo a number of actions, you can `Redo' them (or some",
   "of them; see Redo). However, if you take any new actions after",
   "having `Undo'ne some, you can no longer `Redo' those `Undo'ne actions.",
-  "See Redo",
+  "See Redo.",
 };
 
 const char * const UNLOADMACROS_HELP[12] = {
@@ -1614,6 +1596,58 @@ const char * const UNLOADMACROS_HELP[12] = {
   "version available.",
 };
 
+const char * const UTF8_HELP[20] = {
+  "Syntax: UTF8 [0|1]",
+  "Abbreviation: U8",
+  "",
+  "sets the UTF-8 flag.  When this flag is true, `ne' considers the",
+  "current buffer as UTF-8 coded.  Note that this flag is set",
+  "automatically upon file loading (if possible) if you required automatic",
+  "detection. See UTF8Auto.",
+  "",
+  "   If you invoke `UTF8' with no arguments, it will toggle the flag.  If",
+  "you specify 0 or 1, the flag will be set to false or true,",
+  "respectively. When you try to set this flag, the buffer will be checked",
+  "for UTF-8 compliance, and you will get an error message in case of",
+  "failure. When you try to reset it, the buffer is set to ASCII or",
+  "ISO8859, depending on its content.  A `U' will appear on the status bar",
+  "if the flag is true. Alternatively, an `A' or an `8' will be displayed",
+  "to denote whether the buffer is composed esclusively by US-ASCII",
+  "characters, or also by other 8-bit characters (which are likely to be",
+  "ISO-8859 encoded). Note that each time this command modifies the buffer",
+  "encoding, it also resets the undo buffer.",
+};
+
+const char * const UTF8AUTO_HELP[13] = {
+  "Syntax: UTF8Auto [0|1]",
+  "Abbreviation: U8A",
+  "",
+  "sets the UTF-8 automatic-detection flag.  When this flag is true, `ne'",
+  "will try to guess whether a file just loaded is UTF-8 encoded.",
+  "Moreover, when a non US-ASCII character is inserted in a pure US-ASCII",
+  "buffer, ne will automatically switch to UTF-8. See UTF8. The flag",
+  "is true by default if `ne' detects UTF-8 I/O at startup. See UTF8IO.",
+  "",
+  "   If you invoke `UTF8Auto' with no arguments, it will toggle the flag.",
+  "If you specify 0 or 1, the flag will be set to false or true,",
+  "respectively.",
+};
+
+const char * const UTF8IO_HELP[13] = {
+  "Syntax: UTF8IO [0|1]",
+  "Abbreviation: U8IO",
+  "",
+  "sets the UTF-8 input/output flag.  This flag is set automatically",
+  "depending on your locale setting, and is used to determine whether",
+  "communication with the user (keyboard and terminal) should be UTF-8",
+  "encoded. Normally you would not alter this flag, but sometimes `ne' may",
+  "make the wrong guess (e.g., when you are remotely connected).",
+  "",
+  "   If you invoke `UTF8IO' with no arguments, it will toggle the flag.",
+  "If you specify 0 or 1, the flag will be set to false or true,",
+  "respectively. An `@' will appear on the status bar if the flag is true.",
+};
+
 const char * const VERBOSEMACROS_HELP[17] = {
   "Syntax: VerboseMacros [0|1]",
   "Abbreviation: VM",
