@@ -1,85 +1,102 @@
 #!/usr/bin/perl -w
 
-use strict qw(
-               $enums_h_fname
-               $hash_h_fname
-               $hash_c_fname
-               $help_h_fname
-               $help_c_fname
-               $names_h_fname
-               $names_c_fname
-               $command
-               %commands
-               $hash_table_size
-               $max_hash_table_size
-               @primes
-               @short_hash_table
-               @long_hash_table
-             );
-
+use strict;
 $| = 1;             
-# Todd_Lewis@unc.edu
 
+# Todd_Lewis@unc.edu
+#
 # This program reads the ne.info* files and ne's command names
 # and descriptions from them, and creates relevant parts of
 # ne's source code from this data.  It works with output from
 #     makeinfo (GNU texinfo 3.12) 1.68
 # (and probably other versions too).
 
-$enums_h_fname = "enums.h";
-$hash_h_fname  = "hash.h";
-$hash_c_fname  = "hash.c";
+my $copyright =
+  qq[	Copyright (C) 1993-1998 Sebastiano Vigna 
+	Copyright (C) 1999-2004 Todd M. Lewis and Sebastiano Vigna
 
-$help_h_fname  = "help.h";
-$help_c_fname  = "help.c";
+	This file is part of ne, the nice editor.
 
-$names_h_fname = "names.h";
-$names_c_fname = "names.c";
+	This program is free software; you can redistribute it and/or modify it
+	under the terms of the GNU General Public License as published by the
+	Free Software Foundation; either version 2, or (at your option) any
+	later version.
+	
+	This program is distributed in the hope that it will be useful, but
+	WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+	General Public License for more details.
+	
+	You should have received a copy of the GNU General Public License along
+	with this program; see the file COPYING.  If not, write to the Free
+	Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+	02111-1307, USA.];
 
+my $enums_h_fname = "enums.h";
+my $hash_h_fname  = "hash.h";
+my $hash_c_fname  = "hash.c";
 
-open INFO, "cat ne.info*|" or die "Can't read command names.";
+my $help_h_fname  = "help.h";
+my $help_c_fname  = "help.c";
 
-while ( <INFO> )
+my $names_h_fname = "names.h";
+my $names_c_fname = "names.c";
+
+my @infofiles = glob "ne.info{,-*}";
+
+my ( %commands,
+     $body_enums,
+     $body_names,
+     $hash_table_size,
+     @long_hash_table,
+     @short_hash_table,
+   );
+
+foreach ( @infofiles )
   {
-    chomp;
-    next unless ( m/^(Syntax: )`(([^ ]+).*)'\s*$/ );
-    $command = uc $3;
-    $commands{$command}->{"cmd"}     = "$3";
-    $commands{$command}->{"text"}[0] = "$1$2";
-    
-    # print "---1 found $command\n";
-    
+    open INFO, $_ or die "Can't read command names.";
+
     while ( <INFO> )
       {
-        last if ( substr($_,0,1) eq chr(0x1f) );
         chomp;
-
-        # print "---2 read \"$_\"\n";
-
-        # make info hyperlinks more readable.
-        s/\*note\b/see/g;
-        s/\*Note\b/See/g;
-        s/::\.{0,1}//g;
+        next unless ( m/^(Syntax: )`(([^ ]+).*)'\s*$/ );
+        my $command = uc $3;
+        $commands{$command}->{"cmd"}     = "$3";
+        $commands{$command}->{"text"}[0] = "$1$2";
         
-        if ( m/^(Abbreviation: )`(.+)'/ )
-            {
-              $commands{$command}->{"abbr"} = "$2";
-              push @{$commands{$command}->{"text"}}, "$1$2";
-              # print "---3 pushing $1$2\n";
-            }
-          else
-            {
-              push @{$commands{$command}->{"text"}}, $_;
-              # print "---4 pushing $_\n";
-            }
+        # print "---1 found $command\n";
+        
+        while ( <INFO> )
+          {
+            last if ( substr($_,0,1) eq chr(0x1f) );
+            chomp;
+    
+            # print "---2 read \"$_\"\n";
+    
+            # make info hyperlinks more readable.
+            s/\*note\b/see/g;
+            s/\*Note\b/See/g;
+            s/:://g;
+            
+            if ( m/^(Abbreviation: )`(.+)'/ )
+                {
+                  $commands{$command}->{"abbr"} = "$2";
+                  push @{$commands{$command}->{"text"}}, "$1$2";
+                  # print "---3 pushing $1$2\n";
+                }
+              else
+                {
+                  push @{$commands{$command}->{"text"}}, $_;
+                  # print "---4 pushing $_\n";
+                }
+          }
+        pop @{$commands{$command}->{"text"}}
+            while ($commands{$command}->{"text"}[$#{$commands{$command}->{"text"}}] eq "");
       }
-    pop @{$commands{$command}->{"text"}}
-        while ($commands{$command}->{"text"}[$#{$commands{$command}->{"text"}}] eq "");
-    die "Unexpected end of input on ../doc/ne.info-3." if ( eof INFO );
+    
+    close INFO;
   }
 
-close INFO;
-
 ############################################################################
 # Now the fun part -- We need to generate a hash table big enough to hold
 # all the hashed commands and command abbreviations, without collisions.
@@ -96,11 +113,11 @@ close INFO;
 sub hash
   {
     my ($cmd) = @_;
-    my @cvals = unpack('C*', $cmd );
-    my $h = 0;
+    my @cvals = reverse( unpack('C*', uc $cmd ) );
+    my $h = -1;
     foreach ( @cvals )
       {
-        $h = ($h*26 + ( $_ & 0x1f )) % $hash_table_size;
+        $h = ($h * 31 +  $_ ) % $hash_table_size;
       }
     return $h;
   }
@@ -116,14 +133,13 @@ sub hash
 # each entry of composite, we leave undefined the prime numbers, and
 # we use the (index in array) construct in order to check for primality.
 
-$max_hash_table_size = 6000;
-gen_primes();
+my $max_hash_table_size = 6000;
 gen_hash_tables();
 
 sub gen_primes
   {
     my ($i,$j);
-    @primes = (1) x $max_hash_table_size;    # mark them all prime.
+    my @primes = (1) x $max_hash_table_size;    # mark them all prime.
     for ($i=2; $i<$max_hash_table_size; $i++)
       {
         if ( $primes[$i] )
@@ -134,6 +150,7 @@ sub gen_primes
                 }
             }
       }
+    return @primes;
   }
 
 # Then, starting from 1500 we test in order all prime numbers: we build
@@ -143,7 +160,7 @@ sub gen_primes
 sub gen_hash_tables
   {
     my($cmd,$long,$short,$cmd_index);
-
+    my @primes = gen_primes();
     HTSIZE:
     for ($hash_table_size = 1500;
          $hash_table_size < $max_hash_table_size;
@@ -183,7 +200,7 @@ sub gen_hash_tables
 
 open_out_files();
 
-foreach $cmd ( sort keys %commands )
+foreach my $cmd ( sort keys %commands )
   {
     body_names_h($cmd);
     body_names_c($cmd);
@@ -250,25 +267,7 @@ sub head_names_c
     print NAMES_C <<"_EOT_";
 /* Names and abbreviations of all the commands.
 
-	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2001 Todd M. Lewis and Sebastiano Vigna
-
-	This file is part of ne, the nice editor.
-
-	This program is free software; you can redistribute it and/or modify it
-	under the terms of the GNU General Public License as published by the
-	Free Software Foundation; either version 2, or (at your option) any
-	later version.
-	
-	This program is distributed in the hope that it will be useful, but
-	WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-	General Public License for more details.
-	
-	You should have received a copy of the GNU General Public License along
-	with this program; see the file COPYING.  If not, write to the Free
-	Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-	02111-1307, USA.  */
+$copyright  */
 
 #include "ne.h"
 
@@ -304,7 +303,7 @@ const char ${uccmd}_ABBREV[] = "$commands{$uccmd}->{"abbr"}";
 _EOT_
 
     $body_names .= "  ${uccmd}_NAME,";
-    $body_names .= "\n" if ( (@x = ($body_names =~ m/,/g)) % 4 == 0 ); undef @x;
+    $body_names .= "\n" if ( ( my @x = ($body_names =~ m/,/g)) % 4 == 0 );
   }
   
 sub tail_names_c
@@ -334,25 +333,7 @@ sub head_names_h
     print NAMES_H <<"_EOT_";
 /* Extern's for names and abbreviations of all the commands.
 
-	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2001 Todd M. Lewis and Sebastiano Vigna
-
-	This file is part of ne, the nice editor.
-
-	This program is free software; you can redistribute it and/or modify it
-	under the terms of the GNU General Public License as published by the
-	Free Software Foundation; either version 2, or (at your option) any
-	later version.
-	
-	This program is distributed in the hope that it will be useful, but
-	WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-	General Public License for more details.
-	
-	You should have received a copy of the GNU General Public License along
-	with this program; see the file COPYING.  If not, write to the Free
-	Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-	02111-1307, USA.  */
+$copyright  */
 
 
 _EOT_
@@ -396,26 +377,7 @@ sub head_help_c
     print HELP_C <<"_EOT_";
 /* Help strings.
 
-	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2001 Todd M. Lewis and Sebastiano Vigna
-
-	This file is part of ne, the nice editor.
-
-	This program is free software; you can redistribute it and/or modify it
-	under the terms of the GNU General Public License as published by the
-	Free Software Foundation; either version 2, or (at your option) any
-	later version.
-	
-	This program is distributed in the hope that it will be useful, but
-	WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-	General Public License for more details.
-	
-	You should have received a copy of the GNU General Public License along
-	with this program; see the file COPYING.  If not, write to the Free
-	Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-	02111-1307, USA.  */
-
+$copyright  */
 
 _EOT_
   }
@@ -428,9 +390,9 @@ sub body_help_c
     my $len   = @{$commands{$uccmd}->{"text"}} + 1;
 
     print HELP_C "const char * const ${uccmd}_HELP[$len] = {\n";
-    for $txt ( @{$commands{$uccmd}->{"text"}} )
+    for my $txt ( @{$commands{$uccmd}->{"text"}} )
       {
-        $etxt = $txt;
+        my $etxt = $txt;
         $etxt =~ s#\\#\\\\#g;
         $etxt =~ s#"#\\"#g;
         print HELP_C "  \"$etxt\",\n";
@@ -456,25 +418,7 @@ sub head_help_h
     print HELP_H <<"_EOT_";
 /* Help string externs.
 
-	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2001 Todd M. Lewis and Sebastiano Vigna
-
-	This file is part of ne, the nice editor.
-
-	This program is free software; you can redistribute it and/or modify it
-	under the terms of the GNU General Public License as published by the
-	Free Software Foundation; either version 2, or (at your option) any
-	later version.
-	
-	This program is distributed in the hope that it will be useful, but
-	WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-	General Public License for more details.
-	
-	You should have received a copy of the GNU General Public License along
-	with this program; see the file COPYING.  If not, write to the Free
-	Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-	02111-1307, USA.  */
+$copyright  */
 
 
 _EOT_
@@ -522,7 +466,7 @@ sub body_enums_h
     my $cmd     = $commands{$uccmd}->{"cmd"};
 
     $body_enums .= "  ${uccmd}_A,";  # Count the commas, add '\n' if we've got multiple of 4.
-    $body_enums .= "\n" if ( (@x = ($body_enums =~ m/,/g)) % 4 == 0 ); undef @x;
+    $body_enums .= "\n" if ( (my @x = ($body_enums =~ m/,/g)) % 4 == 0 );
   }
   
 sub tail_enums_h
@@ -547,25 +491,7 @@ sub head_hash_h
     print HASH_H <<"_EOT_";
 /* Header for precompiled hash tables for internal commands.
 
-	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2001 Todd M. Lewis and Sebastiano Vigna
-
-	This file is part of ne, the nice editor.
-
-	This program is free software; you can redistribute it and/or modify it
-	under the terms of the GNU General Public License as published by the
-	Free Software Foundation; either version 2, or (at your option) any
-	later version.
-	
-	This program is distributed in the hope that it will be useful, but
-	WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-	General Public License for more details.
-	
-	You should have received a copy of the GNU General Public License along
-	with this program; see the file COPYING.  If not, write to the Free
-	Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-	02111-1307, USA.  */
+$copyright  */
 
 _EOT_
   }
@@ -605,25 +531,7 @@ sub head_hash_c
     print HASH_C <<"_EOT_";
 /* Precompiled hash tables for internal commands.
 
-	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2001 Todd M. Lewis and Sebastiano Vigna
-
-	This file is part of ne, the nice editor.
-
-	This program is free software; you can redistribute it and/or modify it
-	under the terms of the GNU General Public License as published by the
-	Free Software Foundation; either version 2, or (at your option) any
-	later version.
-	
-	This program is distributed in the hope that it will be useful, but
-	WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-	General Public License for more details.
-	
-	You should have received a copy of the GNU General Public License along
-	with this program; see the file COPYING.  If not, write to the Free
-	Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-	02111-1307, USA.  */
+$copyright  */
 
 
 /* #include "ne.h" */
@@ -650,7 +558,7 @@ sub body_hash_c
 sub tail_hash_c
   {
     print HASH_C "const unsigned char hash_table[HASH_TABLE_SIZE] = {\n";
-    for ($i=0; $i<$hash_table_size; $i++)
+    for (my $i=0; $i<$hash_table_size; $i++)
       {
         print HASH_C "  $long_hash_table[$i],";
         print HASH_C "\n" if ( $i % 20 == 0 );
@@ -658,7 +566,7 @@ sub tail_hash_c
     print HASH_C "};\n\n"; 
     
     print HASH_C "const unsigned char short_hash_table[HASH_TABLE_SIZE] = {\n";
-    for ($i=0; $i<$hash_table_size; $i++)
+    for (my $i=0; $i<$hash_table_size; $i++)
       {
         print HASH_C "  $short_hash_table[$i],";
         print HASH_C "\n" if ( $i % 20 == 0 );
@@ -670,4 +578,3 @@ sub tail_hash_c
 _EOT_
   }
 
-
