@@ -1,7 +1,7 @@
 /* Input class, key bindings and upper casing vector definitions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2001 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2004 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -24,11 +24,11 @@
 #include "ne.h"
 
 
-/* This vector contains all the key bindings. Each entry points to a command
-line to be executed when the corresponding keystroke is input. The index
-correspond to the ASCII code, and to the codes defined in keycodes.h for the
-special keys. Note that it is nonsense to specify a binding for a key whose
-class is not COMMAND. */
+/* This vector contains all key bindings. Each entry points to a command line
+	to be executed when the corresponding keystroke is input. The index
+	correspond to the ASCII code, and to the codes defined in keycodes.h for the
+	special keys. Note that it is nonsense to specify a binding for a key whose
+	class is not COMMAND. */
 
 #ifndef SHELLKEYS
   #define PICK(A,B) A
@@ -91,7 +91,7 @@ const char *key_binding[NUM_KEYS] = {
 	/* Control-meta-letter bindings */
 
 	/* @ a b c d e f g h i j k l m n o */
-	NULL, MOVESOF_ABBREV, PREVWORD_ABBREV, CENTERVIEW_ABBREV, NULL, MOVEEOF_ABBREV, NEXTWORD_ABBREV, GOTOBOOKMARK_ABBREV, NULL, AUTOINDENT_ABBREV, GOTOCOLUMN_ABBREV, SETBOOKMARK_ABBREV, TOLOWER_ABBREV, PLAYONCE_ABBREV, NEXTDOC_ABBREV, OPENCLIP_ABBREV,
+	NULL, MOVESOF_ABBREV, PREVWORD_ABBREV, CENTERVIEW_ABBREV, NEXTDOC_ABBREV, MOVEEOF_ABBREV, NEXTWORD_ABBREV, GOTOBOOKMARK_ABBREV, NULL, AUTOINDENT_ABBREV, GOTOCOLUMN_ABBREV, SETBOOKMARK_ABBREV, TOLOWER_ABBREV, PLAYONCE_ABBREV, OPENNEW_ABBREV, OPENCLIP_ABBREV,
 
 	/* p q r s t u v w x y z [ \ ] ^  _*/
 	PARAGRAPH_ABBREV, QUIT_ABBREV, REDO_ABBREV, SAVECLIP_ABBREV, THROUGH_ABBREV, UNDO_ABBREV, TOUPPER_ABBREV, WORDWRAP_ABBREV, EXIT_ABBREV, DELETEEOL_ABBREV, CRLF_ABBREV, NULL, NULL, NULL, NULL, NULL,
@@ -132,7 +132,7 @@ const char *key_binding[NUM_KEYS] = {
 	/* Prefix-simulated META */
 
 	/* ^- @ a b c d e f g h i j k l m n o */
-	NULL, MOVESOF_ABBREV, PREVWORD_ABBREV, CENTERVIEW_ABBREV, NULL, MOVEEOF_ABBREV, NEXTWORD_ABBREV, GOTOBOOKMARK_ABBREV, NULL, AUTOINDENT_ABBREV, GOTOCOLUMN_ABBREV, SETBOOKMARK_ABBREV, TOLOWER_ABBREV, PLAYONCE_ABBREV, NEXTDOC_ABBREV, OPENCLIP_ABBREV,
+	NULL, MOVESOF_ABBREV, PREVWORD_ABBREV, CENTERVIEW_ABBREV, NEXTDOC_ABBREV, MOVEEOF_ABBREV, NEXTWORD_ABBREV, GOTOBOOKMARK_ABBREV, NULL, AUTOINDENT_ABBREV, GOTOCOLUMN_ABBREV, SETBOOKMARK_ABBREV, TOLOWER_ABBREV, PLAYONCE_ABBREV, OPENNEW_ABBREV, OPENCLIP_ABBREV,
 	
 	/* ^- p q r s t u v w x y z [ \ ] ^  _*/
 	PARAGRAPH_ABBREV, QUIT_ABBREV, REDO_ABBREV, SAVECLIP_ABBREV, THROUGH_ABBREV, UNDO_ABBREV, TOUPPER_ABBREV, WORDWRAP_ABBREV, EXIT_ABBREV, DELETEEOL_ABBREV, CRLF_ABBREV, NULL, NULL, NULL, NULL, NULL,
@@ -141,13 +141,13 @@ const char *key_binding[NUM_KEYS] = {
 	NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
 
 	/* @ A B C D E F G H I J K L M N O */
-	NULL, MOVESOF_ABBREV, PREVWORD_ABBREV, CENTERVIEW_ABBREV, NULL, MOVEEOF_ABBREV, NEXTWORD_ABBREV, GOTOBOOKMARK_ABBREV, NULL, AUTOINDENT_ABBREV, GOTOCOLUMN_ABBREV, SETBOOKMARK_ABBREV, TOLOWER_ABBREV, PLAYONCE_ABBREV, NEXTDOC_ABBREV, OPENCLIP_ABBREV,
+	NULL, MOVESOF_ABBREV, PREVWORD_ABBREV, CENTERVIEW_ABBREV, NEXTDOC_ABBREV, MOVEEOF_ABBREV, NEXTWORD_ABBREV, GOTOBOOKMARK_ABBREV, NULL, AUTOINDENT_ABBREV, GOTOCOLUMN_ABBREV, SETBOOKMARK_ABBREV, TOLOWER_ABBREV, PLAYONCE_ABBREV, OPENNEW_ABBREV, OPENCLIP_ABBREV,
 
 	/* P Q R S T U V W X Y Z [ \ ] ^  _*/
 	PARAGRAPH_ABBREV, QUIT_ABBREV, REDO_ABBREV, SAVECLIP_ABBREV, THROUGH_ABBREV, UNDO_ABBREV, TOUPPER_ABBREV, WORDWRAP_ABBREV, EXIT_ABBREV, DELETEEOL_ABBREV, CRLF_ABBREV, NULL, NULL, NULL, NULL, NULL,
 
 	/* @ a b c d e f g h i j k l m n o */
-	NULL, MOVESOF_ABBREV, PREVWORD_ABBREV, CENTERVIEW_ABBREV, NULL, MOVEEOF_ABBREV, NEXTWORD_ABBREV, GOTOBOOKMARK_ABBREV, NULL, AUTOINDENT_ABBREV, GOTOCOLUMN_ABBREV, SETBOOKMARK_ABBREV, TOLOWER_ABBREV, PLAYONCE_ABBREV, NEXTDOC_ABBREV, OPENCLIP_ABBREV,
+	NULL, MOVESOF_ABBREV, PREVWORD_ABBREV, CENTERVIEW_ABBREV, NEXTDOC_ABBREV, MOVEEOF_ABBREV, NEXTWORD_ABBREV, GOTOBOOKMARK_ABBREV, NULL, AUTOINDENT_ABBREV, GOTOCOLUMN_ABBREV, SETBOOKMARK_ABBREV, TOLOWER_ABBREV, PLAYONCE_ABBREV, OPENNEW_ABBREV, OPENCLIP_ABBREV,
 
 	/* p q r s t u v w x y z [ \ ] ^  _*/
 	PARAGRAPH_ABBREV, QUIT_ABBREV, REDO_ABBREV, SAVECLIP_ABBREV, THROUGH_ABBREV, UNDO_ABBREV, TOUPPER_ABBREV, WORDWRAP_ABBREV, EXIT_ABBREV, DELETEEOL_ABBREV, CRLF_ABBREV, NULL, NULL, NULL, NULL, NULL
@@ -157,9 +157,9 @@ const char *key_binding[NUM_KEYS] = {
 
 
 
-/* This vector holds, for each extended key code, its input class. */
+/* This vector holds, for each ISO-8859-1 key code, its input class. */
 
-const input_class char_class[NUM_KEYS] = {
+const input_class char_class[256] = {
 
 	/* Control-letter classes */
 
@@ -194,32 +194,10 @@ const input_class char_class[NUM_KEYS] = {
 	ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA,
 	ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA,
 	ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA,
-	ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA,
-
-
-	IGNORE, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, IGNORE, IGNORE, IGNORE, IGNORE, IGNORE,
-	COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, IGNORE, IGNORE, IGNORE, IGNORE, IGNORE, IGNORE, IGNORE,
-	COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, IGNORE, IGNORE, COMMAND, COMMAND, COMMAND, IGNORE, IGNORE, IGNORE, IGNORE, IGNORE,
-	IGNORE, IGNORE, IGNORE, IGNORE, IGNORE, IGNORE, IGNORE, IGNORE, IGNORE, IGNORE, IGNORE, IGNORE, IGNORE, IGNORE, IGNORE, IGNORE,
-
-	COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND,
-	COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND,
-	COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND,
-	COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND,
-
-	COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND,
-	COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND,
-	COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND,
-	COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND,
-
-	COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND,
-	COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND,
-	COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND,
-	COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND, COMMAND
+	ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA, ALPHA
 };
 
 
-
 const char meta_prefixed[128][3] = {
 	{ '\x1b', 0 },
 	{ '\x1b', 1 },
