@@ -1,7 +1,7 @@
 /* Menu handling function. Includes also key and menu configuration parsing.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2001 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2004 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -169,7 +169,8 @@ static const menu_item extras_item[] =
 		{ "Center View [C", CENTERVIEW_ABBREV },
 		{ "ToUpper     [V", TOUPPER_ABBREV },
 		{ "ToLower     [L", TOLOWER_ABBREV },
-		{ "Capitalize    ", CAPITALIZE_ABBREV }
+		{ "Capitalize    ", CAPITALIZE_ABBREV },
+		{ "UTF-8         ", UTF8_ABBREV }
 	};
 
 
@@ -184,8 +185,8 @@ PICK(   "Prev Page     ^P", PREVPAGE_ABBREV    , "Page Up       ^P", PAGEUP_ABBR
 PICK(   "Next Page     ^N", NEXTPAGE_ABBREV    , "Page Down     ^N", PAGEDOWN_ABBREV)
 PICK(   "Page Up         ", PAGEUP_ABBREV      , "Prev Page       ", PREVPAGE_ABBREV)
 PICK(   "Page Down       ", PAGEDOWN_ABBREV    , "Next Page       ", NEXTPAGE_ABBREV)
-		{ "Start Of File [A", MOVESOL_ABBREV },
-		{ "End Of File   [E", MOVEEOL_ABBREV },
+		{ "Start Of File [A", MOVESOF_ABBREV },
+		{ "End Of File   [E", MOVEEOF_ABBREV },
 		{ "Start Of Line ^A", MOVESOL_ABBREV },
 		{ "End Of Line   ^E", MOVEEOL_ABBREV },
 		{ "Top Of Screen   ", MOVETOS_ABBREV },
@@ -220,7 +221,7 @@ static const menu_item prefs_item[] =
 
 static menu def_menus[DEF_MENU_NUM] = {
 	{
-		sizeof(file_item)/sizeof(menu_item),
+		sizeof(file_item) / sizeof(menu_item),
 		1,
 		12,
 		0,
@@ -228,7 +229,7 @@ static menu def_menus[DEF_MENU_NUM] = {
 		file_item
 	},
 	{
-		sizeof(documents_item)/sizeof(menu_item),
+		sizeof(documents_item) / sizeof(menu_item),
 		6,
 		12,
 		0,
@@ -236,7 +237,7 @@ static menu def_menus[DEF_MENU_NUM] = {
 		documents_item
 	},
 	{
-		sizeof(edit_item)/sizeof(menu_item),
+		sizeof(edit_item) / sizeof(menu_item),
 		16,
 		14,
 		0,
@@ -244,7 +245,7 @@ static menu def_menus[DEF_MENU_NUM] = {
 		edit_item
 	},
 	{
-		sizeof(search_item)/sizeof(menu_item),
+		sizeof(search_item) / sizeof(menu_item),
 		21,
 		17,
 		0,
@@ -252,7 +253,7 @@ static menu def_menus[DEF_MENU_NUM] = {
 		search_item
 	},
 	{
-		sizeof(macros_item)/sizeof(menu_item),
+		sizeof(macros_item) / sizeof(menu_item),
 		28,
 		17,
 		0,
@@ -260,7 +261,7 @@ static menu def_menus[DEF_MENU_NUM] = {
 		macros_item
 	},
 	{
-		sizeof(extras_item)/sizeof(menu_item),
+		sizeof(extras_item) / sizeof(menu_item),
 		35,
 		14,
 		0,
@@ -268,7 +269,7 @@ static menu def_menus[DEF_MENU_NUM] = {
 		extras_item
 	},
 	{
-		sizeof(navigation_item)/sizeof(menu_item),
+		sizeof(navigation_item) / sizeof(menu_item),
 		42,
 		16,
 		0,
@@ -276,7 +277,7 @@ static menu def_menus[DEF_MENU_NUM] = {
 		navigation_item
 	},
 	{
-		sizeof(prefs_item)/sizeof(menu_item),
+		sizeof(prefs_item) / sizeof(menu_item),
 		53,
 		16,
 		0,
@@ -299,17 +300,17 @@ static menu *menus = def_menus;
 
 
 #ifdef DONT_COMPILE_THIS_PART
-int dump_config( void )
+int dump_config(void)
   {
     int menu, item, key;
     FILE *f;
     
-    if (!(f=fopen("dump_config","w")) ) return ERROR;
+    if (!(f = fopen("dump_config","w")) ) return ERROR;
     
-    for (menu=0; menu<menu_num; menu++)
+    for (menu = 0; menu < menu_num; menu++)
       {
         fprintf(f,"%s      \"%s\"\n", MENU_KEYWORD, menus[menu].text );
-        for (item=0; item<menus[menu].item_num; item++)
+        for (item = 0; item < menus[menu].item_num; item++)
           {
             fprintf(f,"     %s \"%s\" \"%s\"\n", ITEM_KEYWORD,
                                                 menus[menu].items[item].text,
@@ -318,7 +319,7 @@ int dump_config( void )
         fprintf(f,"\n");
       }
     
-    for (key=0; key<NUM_KEYS; key++)
+    for (key = 0; key < NUM_KEYS; key++)
       {
         if ( key_binding[key] && key_binding[key][0] )
           {
@@ -330,74 +331,80 @@ int dump_config( void )
   }
 #endif
   
-static void draw_cur_item(int n) {
-	move_cursor(1+menus[n].cur_item, menus[n].xpos);
-	if (!cur_buffer->opt.fast_gui && standout_ok) output_chars(menus[n].items[menus[n].cur_item].text, menus[n].width-(cursor_on_off_ok ? 0 : 1));
+static void draw_cur_item(const int n) {
+	move_cursor(menus[n].cur_item + 1, menus[n].xpos);
+	if (!cur_buffer->opt.fast_gui && standout_ok) output_chars(menus[n].items[menus[n].cur_item].text, menus[n].width - (cursor_on_off_ok ? 0 : 1), TRUE);
 }
 
 
-static void undraw_cur_item(int n) {
+static void undraw_cur_item(const int n) {
 	if (!cur_buffer->opt.fast_gui && standout_ok)  {
 		standout_on();
-		move_cursor(1+menus[n].cur_item, menus[n].xpos);
-		output_chars(menus[n].items[menus[n].cur_item].text, menus[n].width-(cursor_on_off_ok ? 0 : 1));
+		move_cursor(1 + menus[n].cur_item, menus[n].xpos);
+		output_chars(menus[n].items[menus[n].cur_item].text, menus[n].width - (cursor_on_off_ok ? 0 : 1), TRUE);
 		standout_off();
 	}
 }
 
 
-/* This function draws a given menu. It also draws the current menu item. */
+/* Draws a given menu. It also draws the current menu item. */
 
-static void draw_menu(int n) {
+static void draw_menu(const int n) {
 	int i;
 
 	assert(menus[n].xpos > 0);
 
+	if (menus[n].cur_item + 1 + (standout_ok == 0) >= ne_lines - 1) menus[n].cur_item = 0;
+
 	move_cursor(0, menus[n].xpos);
-	output_string(menus[n].text);
+	output_string(menus[n].text, TRUE);
 
-	for(i=0; i<menus[n].item_num; i++) {
-		move_cursor(i+1, menus[n].xpos-1);
+	for(i = 0; i < menus[n].item_num; i++) {
+		if (i + 1 + (standout_ok == 0) >= ne_lines - 1) break;
 
-		if (!standout_ok) output_chars("|", 1);
+		move_cursor(i + 1, menus[n].xpos - 1);
+
+		if (!standout_ok) output_chars("|", 1, FALSE);
 
 		standout_on();
-		output_chars(NULL, 1);
-		output_string(menus[n].items[i].text);
-		output_chars(NULL, 1);
+		output_char(' ', FALSE);
+		output_string(menus[n].items[i].text, TRUE);
+		output_char(' ', FALSE);
 		standout_off();
 
-		if (!standout_ok) output_chars("|", 1);
+		if (!standout_ok) output_chars("|", 1, FALSE);
 	}
 
 	if (!standout_ok) {
-		move_cursor(i+1, menus[n].xpos-1);
-		for(i=0; i<menus[n].width+(standout_ok ? MENU_EXTRA : MENU_NOSTANDOUT_EXTRA); i++) output_chars("-", 1);
+		move_cursor(i + 1, menus[n].xpos - 1);
+		for(i = 0; i < menus[n].width + (standout_ok ? MENU_EXTRA : MENU_NOSTANDOUT_EXTRA); i++) output_chars("-", 1, FALSE);
 	}
 
 	draw_cur_item(n);
 }
 
 
-/* This function undraws a menu. This is obtained by refreshing part of the
-screen via mvaddstrn(). */
+/* Undraws a menu. This is obtained by refreshing part of the
+screen via output_line_desc(). */
 
-static void undraw_menu(int n) {
+static void undraw_menu(const int n) {
 	int i;
 	line_desc *ld = cur_buffer->top_line_desc;
 
+
 	standout_on();
 	move_cursor(0, menus[n].xpos);
-	output_string(menus[n].text);
+	output_string(menus[n].text, TRUE);
 	standout_off();
 
-	for(i=1; i<=menus[n].item_num+(standout_ok ? 0 : 1); i++) {
+	for(i = 1; i <= menus[n].item_num + (standout_ok == 0); i++) {
+		if (i >= ne_lines - 1) break;
 		if (ld->ld_node.next->next) {
 			ld = (line_desc *)ld->ld_node.next;
-			mvaddstrn(i, menus[n].xpos-1, ld, cur_buffer->win_x+menus[n].xpos-1, menus[n].width+(standout_ok ? MENU_EXTRA : MENU_NOSTANDOUT_EXTRA), cur_buffer->opt.tab_size, FALSE);
+			output_line_desc(i, menus[n].xpos - 1, ld, cur_buffer->win_x + menus[n].xpos - 1, menus[n].width + (standout_ok ? MENU_EXTRA : MENU_NOSTANDOUT_EXTRA), cur_buffer->opt.tab_size, FALSE, cur_buffer->encoding == ENC_UTF8);
 		}
 		else {
-			move_cursor(i, menus[n].xpos-1);
+			move_cursor(i, menus[n].xpos - 1);
 			clear_to_eol();
 		}
 	}
@@ -407,17 +414,19 @@ static void undraw_menu(int n) {
 
 static void draw_next_item(void) {
 	undraw_cur_item(current_menu);
-	menus[current_menu].cur_item = (menus[current_menu].cur_item+1) % menus[current_menu].item_num;
+	menus[current_menu].cur_item = (menus[current_menu].cur_item + 1) % menus[current_menu].item_num;
+	if (menus[current_menu].cur_item + 1 + (standout_ok == 0) >= ne_lines - 1) menus[current_menu].cur_item = 0;
 	draw_cur_item(current_menu);
 }
 
 static void draw_prev_item(void) {
 	undraw_cur_item(current_menu);
-	if (--(menus[current_menu].cur_item) < 0) menus[current_menu].cur_item = menus[current_menu].item_num-1;
+	if (--(menus[current_menu].cur_item) < 0) menus[current_menu].cur_item = menus[current_menu].item_num - 1;
+	if (menus[current_menu].cur_item + 1 + (standout_ok == 0) >= ne_lines - 1) menus[current_menu].cur_item = ne_lines - 3 - (standout_ok == 0) ;
 	draw_cur_item(current_menu);
 }
 
-static void draw_item(int item) {
+static void draw_item(const int item) {
 	undraw_cur_item(current_menu);
 	menus[current_menu].cur_item = item;
 	draw_cur_item(current_menu);
@@ -426,23 +435,27 @@ static void draw_item(int item) {
 static void draw_next_menu(void) {
 	undraw_menu(current_menu);
 	current_menu = ++current_menu % menu_num;
+	if (menus[current_menu].xpos >= ne_columns) current_menu = 0;
 	draw_menu(current_menu);
 }
 
 static void draw_prev_menu(void) {
 	undraw_menu(current_menu);
-	if (--current_menu < 0) current_menu = menu_num-1;
+	if (--current_menu < 0) current_menu = menu_num - 1;
+	while(menus[current_menu].xpos >= ne_columns) current_menu--;
 	draw_menu(current_menu);
 }
 
-int search_menu_title(int n, int c) {
+int search_menu_title(int n, const int c) {
 
 	int i;
 
-	for(i=0; i<menu_num-1; i++)
-		if (menus[++n % menu_num].text[0] == c) return(n % menu_num);
+	for(i = 0; i < menu_num - 1; i++) {
+		if (menus[++n % menu_num].xpos >= ne_columns) continue;
+		if (menus[n % menu_num].text[0] == c) return n % menu_num;
+	}
 
-	return(-1);
+	return -1;
 }
 
 int search_menu_item(int n, int c) {
@@ -451,15 +464,17 @@ int search_menu_item(int n, int c) {
 
 	c = toupper(c);
 
-	for(i=0, j=menus[n].cur_item; i<menus[n].item_num-1; i++)
-		if (menus[n].items[++j % menus[n].item_num].text[0] == c) return(j % menus[n].item_num);
+	for(i = 0, j = menus[n].cur_item; i < menus[n].item_num - 1; i++) {
+		if (++j % menus[n].item_num + 1 + (standout_ok == 0) >= ne_lines - 1) continue;
+		if (menus[n].items[++j % menus[n].item_num].text[0] == c) return j % menus[n].item_num;
+	}
 
-	return(-1);
+	return -1;
 }
 
 
 
-static void item_search(int c) {
+static void item_search(const int c) {
 	int new_item;
 
 	if (c >= 'a' && c <= 'z') {
@@ -484,13 +499,13 @@ static void draw_first_menu(void) {
 	move_cursor(0,0);
 
 	standout_on();
-	if (standout_ok) cursor_off();
+	if (!cur_buffer->opt.fast_gui && standout_ok) cursor_off();
 
-	while(i<ne_columns) {
-		output_chars(NULL, 1);
+	while(i < ne_columns) {
+		output_char(' ', FALSE);
 		i++;
 		if (n < menu_num) {
-			output_string(menus[n].text);
+			output_string(menus[n].text, TRUE);
 			i += strlen(menus[n].text);
 			n++;
 		}
@@ -498,6 +513,7 @@ static void draw_first_menu(void) {
 
 	if (standout_ok) standout_off();
 
+	if (menus[current_menu].xpos >= ne_columns) current_menu = 0;
 	draw_menu(current_menu);
 }
 
@@ -510,22 +526,22 @@ static void undraw_last_menu(void) {
 
 static void do_menu_action(void) {
 	undraw_last_menu();
-	execute_command_line(cur_buffer, menus[current_menu].items[menus[current_menu].cur_item].command_line);
+	print_error(execute_command_line(cur_buffer, menus[current_menu].items[menus[current_menu].cur_item].command_line));
 }
 
 
 
-/* showing_msg tells draw_status_bar() that a message is currently shown, and should be
-cancelled only on the next refresh. Bar gone says that the status bar doesn't exists any
-longer, so we have to rebuild it entirely. */
+/* showing_msg tells draw_status_bar() that a message is currently shown, and
+	should be cancelled only on the next refresh. Bar gone says that the status
+	bar doesn't exists any longer, so we have to rebuild it entirely. */
 
 static int showing_msg;
 static int bar_gone = TRUE;
 
 
 
-/* This function resets the status bar. It does not perform the refresh, just
-sets bar_gone to TRUE. */
+/* Resets the status bar. It does not perform the refresh, just sets bar_gone
+	to TRUE. */
 
 void reset_status_bar(void) {
 	bar_gone = TRUE;
@@ -541,11 +557,11 @@ the flags are updated the cursor does not need to be moved after
 printing the numbers (an operation which usually needs the output
 of several characters). */
 
-char *gen_flag_string(buffer *b) {
+char *gen_flag_string(const buffer * const b) {
 
 	static char string[MAX_FLAG_STRING_SIZE];
-	int i = 0;
-	char ch = b->cur_pos < b->cur_line_desc->line_len ? b->cur_line_desc->line[b->cur_pos] : 0;
+	int i = 0, j;
+	int ch = b->cur_pos < b->cur_line_desc->line_len ? (b->encoding == ENC_UTF8 ? utf8char(&b->cur_line_desc->line[b->cur_pos]) : b->cur_line_desc->line[b->cur_pos]) : -1;
 
 	string[i++] = ' ';
 	string[i++] = b->opt.insert         ? 'i' : '-';
@@ -563,28 +579,45 @@ char *gen_flag_string(buffer *b) {
 	string[i++] = b->recording          ? 'R' : '-';
 	string[i++] = b->opt.preserve_cr    ? 'P' : '-';
 	string[i++] = b->is_CRLF            ? 'C' : '-';
-	string[i++] = b->buffer_is_modified ? '*' : '-';
+	string[i++] = b->is_modified        ? '*' : '-';
+	string[i++] = io_utf8               ? '@' : '-';
+	string[i++] = b->encoding != ENC_8_BIT? (b->encoding == ENC_UTF8 ? 'U' : 'A') : '8';
 
 	if (b->opt.hex_code && !b->opt.fast_gui) {
 		string[i++] = ' ';
-		string[i++] = "0123456789abcdef"[(ch >> 4) & 0x0f];
-		string[i++] = "0123456789abcdef"[ch & 0x0f];
+		if (ch > 0xFFFF) {
+			string[i++] = "0123456789abcdef"[(ch >> 28) & 0x0f];
+			string[i++] = "0123456789abcdef"[(ch >> 24) & 0x0f];
+			string[i++] = "0123456789abcdef"[(ch >> 20) & 0x0f];
+			string[i++] = "0123456789abcdef"[(ch >> 16) & 0x0f];
+		}
+		else for(j = 0; j < 4; j++) string[i++] = ' ';
+		if (ch > 0xFF) {
+			string[i++] = "0123456789abcdef"[(ch >> 12) & 0x0f];
+			string[i++] = "0123456789abcdef"[(ch >> 8) & 0x0f];
+		}
+		else for(j = 0; j < 2; j++) string[i++] = ' ';
+		if (ch > -1) {
+		   string[i++] = "0123456789abcdef"[(ch >> 4) & 0x0f];
+		   string[i++] = "0123456789abcdef"[ch & 0x0f];
+		}
+		else for(j = 0; j < 2; j++) string[i++] = ' ';
 	}
 
 	string[i] = 0;
 
-	assert(i<MAX_FLAG_STRING_SIZE);
+	assert(i < MAX_FLAG_STRING_SIZE);
 
-	return(string);
+	return string;
 }
 
 
 
-/* This function draws the status bar. If showing_msg is TRUE, it is set to
-FALSE, bar_gone is set to TRUE and the update is deferred to the next
-call. If the bar is not completely gone, we try to just update the line
-and column numbers, and the flags. The function keeps track internally of
-their last values, so that unnecessary printing is avoided. */
+/* Draws the status bar. If showing_msg is TRUE, it is set to FALSE, bar_gone
+	is set to TRUE and the update is deferred to the next call. If the bar is
+	not completely gone, we try to just update the line and column numbers, and
+	the flags. The function keeps track internally of their last values, so that
+	unnecessary printing is avoided. */
 
 
 void draw_status_bar(void) {
@@ -594,7 +627,7 @@ void draw_status_bar(void) {
 	static int x = -1, y = -1;
 
 	char *p;
-	int cur_pos, len;
+	int len;
 	int i;
 
 	if (showing_msg) {
@@ -604,20 +637,25 @@ void draw_status_bar(void) {
 	}
 
 	if (!bar_gone && cur_buffer->opt.status_bar) {
-		if (!cur_buffer->opt.fast_gui && standout_ok) standout_on();
+		/* This is the space occupied up to "L:", included. */
+		const int offset = cur_buffer->opt.fast_gui || !standout_ok ? 5: 3;
+		const int update_linecols = y != cur_buffer->cur_line || x != cur_buffer->win_x + cur_buffer->cur_x;
 
-		if (y != cur_buffer->cur_line || x != cur_buffer->win_x+cur_buffer->cur_x) {
-			x = cur_buffer->win_x+cur_buffer->cur_x;
-			y = cur_buffer->cur_line;
-			i = sprintf(bar_buffer, "%6d C:%6d", y+1, x+1);
-			move_cursor(ne_lines-1, cur_buffer->opt.fast_gui || !standout_ok ? 5: 3);
-			output_chars(bar_buffer, i);
+		if (!cur_buffer->opt.fast_gui && standout_ok) standout_on();
+		
+		x = cur_buffer->win_x + cur_buffer->cur_x;
+		y = cur_buffer->cur_line;
+		i = sprintf(bar_buffer, "%8d C:%8d %3d", y + 1, x + 1, ((y + 1) * 100) / cur_buffer->num_lines);
+
+		if (update_linecols) {
+			move_cursor(ne_lines - 1, offset);
+			output_chars(bar_buffer, i, TRUE);
 		}
 
 		if (strcmp(flag_string, p = gen_flag_string(cur_buffer))) {
 			strcpy(flag_string, p);
-			move_cursor(ne_lines-1, cur_buffer->opt.fast_gui || !standout_ok ? 20: 18);
-			output_string(flag_string);
+			move_cursor(ne_lines - 1, i + offset + 2);
+			output_string(flag_string, TRUE);
 		}
 
 		if (!cur_buffer->opt.fast_gui && standout_ok) standout_off();
@@ -626,49 +664,45 @@ void draw_status_bar(void) {
 
 
 	if (cur_buffer->opt.status_bar) {
-		move_cursor(ne_lines-1, 0);
+		move_cursor(ne_lines - 1, 0);
 		if (!cur_buffer->opt.fast_gui && standout_ok) standout_on();
 
 		strcpy(flag_string, gen_flag_string(cur_buffer));
 
-		x = cur_buffer->win_x+cur_buffer->cur_x;
+		x = cur_buffer->win_x + cur_buffer->cur_x;
 		y = cur_buffer->cur_line;
 
-		len = sprintf(bar_buffer, cur_buffer->opt.fast_gui || !standout_ok ? ">> L:%6ld C:%6ld%s " : " L:%6ld C:%6ld%s ", y+1, x+1, flag_string);
-		if (len > ne_columns) len = ne_columns;
+		len = sprintf(bar_buffer, cur_buffer->opt.fast_gui || !standout_ok ? ">> L:%8d C:%8d %3d%% %s " : " L:%8d C:%8d %3d%% %s ", y + 1, x + 1, ((y + 1) * 100) / cur_buffer->num_lines, flag_string);
 
-		move_cursor(ne_lines-1, 0);
+		move_cursor(ne_lines - 1, 0);
+		output_chars(bar_buffer, len, TRUE);
 
-		output_chars(bar_buffer, len);
-		cur_pos = len;
+		if (len < ne_columns - 1) {
+			if (cur_buffer->filename) {
+	 			/* This is a bit complicated because we have to compute the width of the filename first, and then 
+				discard initial characters until the remaning part will fit. */
 
-		if (cur_buffer->filename ) {
-			len = strlen(cur_buffer->filename);
-			if ( cur_pos < ne_columns-1) {
-				if (cur_pos+len >= ne_columns-1) {
-					int tlen = ne_columns-1-cur_pos;
-					output_chars(&cur_buffer->filename[len-tlen],tlen);
-					len = tlen;
-					}
-				else {
-					output_chars(cur_buffer->filename, len);
-					}
-				cur_pos += len;
+				const int encoding = detect_encoding(cur_buffer->filename, strlen(cur_buffer->filename));
+				int pos = 0, width = get_string_width(cur_buffer->filename, strlen(cur_buffer->filename), encoding);
+
+				while(width > ne_columns - 1 - len) {
+					width -= get_char_width(&cur_buffer->filename[pos], encoding);
+					pos = next_pos(cur_buffer->filename, pos, encoding);
 				}
+
+				output_string(cur_buffer->filename + pos, encoding == ENC_UTF8);
 			}
-			else {
-				output_chars(UNNAMED_NAME, 9);
-				cur_pos += 9;
-			}
-      
+			else output_string(UNNAMED_NAME, FALSE);
+		}
+
 		if (!cur_buffer->opt.fast_gui && standout_ok) {
-			output_chars(NULL, ne_columns-cur_pos);
+			output_spaces(ne_columns);
 			standout_off();
 		}
 		else clear_to_eol();
 	}
 	else if (bar_gone) {
-		move_cursor(ne_lines-1, 0);
+		move_cursor(ne_lines - 1, 0);
 		clear_to_eol();
 	}
 
@@ -677,21 +711,21 @@ void draw_status_bar(void) {
 
 
 
-/* This function prints a message over the status bar. It also sets
-showing_msg and bar_gone. */
+/* Prints a message over the status bar. It also sets showing_msg and
+	bar_gone. */
 
-void print_message(const char *message) {
+void print_message(const char * const message) {
 
-	move_cursor(ne_lines-1, 0);
+	move_cursor(ne_lines - 1, 0);
 
 	if (cur_buffer->opt.fast_gui || !standout_ok || !cur_buffer->opt.status_bar) {
 		clear_to_eol();
-		output_string(message);
+		output_string(message, TRUE);
 	}
 	else {
 		standout_on();
-		output_string(message);
-		output_chars(NULL, ne_columns-strlen(message));
+		output_string(message, TRUE);
+		output_spaces(ne_columns - strlen(message));
 		standout_off();
 	}
 
@@ -702,11 +736,11 @@ void print_message(const char *message) {
 
 
 
-/* This function prints an error on the status bar. error_num is a global
-error code. The function returns the error code passed, and does not do
-anything if the error code is OK or ERROR. */
+/* Prints an error on the status bar. error_num is a global error code. The
+	function returns the error code passed, and does not do anything if the
+	error code is OK or ERROR. */
 
-int print_error(int error_num) {
+int print_error(const int error_num) {
 
 	assert(error_num < ERROR_COUNT);
 
@@ -714,16 +748,16 @@ int print_error(int error_num) {
 		print_message(error_msg[error_num]);
 		ring_bell();
 	}
-	return(error_num);
+	return error_num;
 }
 
 
 
-/* This function prints an information on the status bar. info_num is a global
-information code. Note that no beep is generated. */
+/* Prints an information on the status bar. info_num is a global information
+	code. Note that no beep is generated. */
 
 
-void print_info(int info_num) {
+void print_info(const int info_num) {
 
 	assert(info_num < INFO_COUNT);
 
@@ -732,97 +766,102 @@ void print_info(int info_num) {
 
 
 
-/* This function handles the menu system: it displays the menus, parses the
-keyboard input, and eventually executes the correct command line. Note that
-we support ':' for going to the command line, alphabetic search (upper case
-for menus, lower case for items) and the cursor movement keys (by line,
-character, page). Note also the all other actions are executed, so that you
-can use shortcuts while using menus. */
+/* Handles the menu system: it displays the menus, parses the keyboard input,
+	and eventually executes the correct command line. Note that we support ':'
+	for going to the command line, alphabetic search (upper case for menus,
+	lower case for items) and the cursor movement keys (by line, character,
+	page). Note also the all other actions are executed, so that you can use
+	shortcuts while using menus. */
 
 void handle_menus(void) {
 
 	input_class ic;
 	action a;
 	int c, n;
-	char  *p;
+	unsigned char  *p;
 
 	draw_first_menu();
 
 	while(TRUE) {
-		while((ic = char_class[(c = get_key_code()) & 0x1FF]) == IGNORE);
+		do c = get_key_code(); while((ic = CHAR_CLASS(c)) == IGNORE);
 
 		switch(ic) {
-			case ALPHA:
-				if (c == ':') {
+		case INVALID:
+			ring_bell();
+			break;
+			
+		case ALPHA:
+			if (c == ':') {
+				undraw_last_menu();
+				do_action(cur_buffer, EXEC_A, -1, NULL);
+				return;
+			}
+			item_search(c);
+			break;
+			
+		case RETURN:
+			do_menu_action();
+			return;
+			
+		case COMMAND:
+			if (c < 0) c = -c - 1;
+			if ((a = parse_command_line(key_binding[c], &n, &p, FALSE))>=0) {
+				switch(a) {
+				case MOVELEFT_A:
+					draw_prev_menu();
+					break;
+
+				case MOVERIGHT_A:
+					draw_next_menu();
+					break;
+
+				case LINEUP_A:
+					draw_prev_item();
+					break;
+
+				case LINEDOWN_A:
+					draw_next_item();
+					break;
+
+				case PREVPAGE_A:
+					draw_item(0);
+					break;
+
+				case NEXTPAGE_A:
+					draw_item(menus[current_menu].item_num - 1);
+					break;
+
+				case ESCAPE_A:
 					undraw_last_menu();
-					do_action(cur_buffer, EXEC_A, -1, NULL);
+					return;
+
+				default:
+					undraw_last_menu();
+					do_action(cur_buffer, a, n, p);
 					return;
 				}
-				item_search(c);
-				break;
+			}
+			break;
 
-			case RETURN:
-				do_menu_action();
-				return;
-
-			case COMMAND:
-				if ((a = parse_command_line(key_binding[c], &n, &p, FALSE))>=0) {
-					switch(a) {
-						case MOVELEFT_A:
-							draw_prev_menu();
-							break;
-
-						case MOVERIGHT_A:
-							draw_next_menu();
-							break;
-
-						case LINEUP_A:
-							draw_prev_item();
-							break;
-
-						case LINEDOWN_A:
-							draw_next_item();
-							break;
-
-						case PREVPAGE_A:
-							draw_item(0);
-							break;
-
-						case NEXTPAGE_A:
-							draw_item(menus[current_menu].item_num-1);
-							break;
-
-						case ESCAPE_A:
-							undraw_last_menu();
-							return;
-
-						default:
-							undraw_last_menu();
-							do_action(cur_buffer, a, n, p);
-							return;
-					}
-				}
-				break;
-
-			default:
-				break;
+		default:
+			break;
 		}
 
 	}
 }
 
-static void error_in_menu_configuration(int line, char  *s) {
+static void error_in_menu_configuration(const int line, const char * const s) {
 
 	fprintf(stderr, "Error in menu configuration file at line %d: %s\n", line, s);
 	exit(0);
 }
 
 
-void get_menu_configuration(char *menu_conf_name) {
+void get_menu_configuration(const char * menu_conf_name) {
 
 	char  *prefs_dir, *menu_conf;
 	char_stream *cs;
-	char  *p;
+	unsigned char  *p;
 	int pass, cur_menu, cur_item, num_items_in_menu, line;
 	menu *new_menus;
 	menu_item *new_items;
@@ -830,19 +869,19 @@ void get_menu_configuration(char *menu_conf_name) {
 	if (!menu_conf_name) menu_conf_name = MENU_CONF_NAME;
 
 	if (prefs_dir = exists_prefs_dir()) {
-		if (menu_conf = malloc(strlen(prefs_dir)+strlen(menu_conf_name)+1)) {
+		if (menu_conf = malloc(strlen(prefs_dir) + strlen(menu_conf_name) + 1)) {
 			strcat(strcpy(menu_conf, prefs_dir), menu_conf_name);
 
 			if ((cs = load_stream(NULL, menu_conf_name, FALSE)) || (cs = load_stream(NULL, menu_conf, FALSE))) {
 
-				for(pass=0; pass<2; pass++) {
+				for(pass = 0; pass < 2; pass++) {
 
 					p = cs->stream;
 					line = 1;
 					cur_menu = -1;
 					cur_item = num_items_in_menu = 0;
 
-					while(p-cs->stream<cs->len) {
+					while(p - cs->stream < cs->len) {
 						if (*p) {
 
 							if (!cmdcmp(MENU_KEYWORD, p)) {
@@ -859,7 +898,7 @@ void get_menu_configuration(char *menu_conf_name) {
 											if (*p) {
 												*p++ = 0;
 												if (cur_menu == 0) new_menus[0].xpos = 1;
-												else new_menus[cur_menu].xpos = new_menus[cur_menu-1].xpos+strlen(new_menus[cur_menu-1].text)+1;
+												else new_menus[cur_menu].xpos = new_menus[cur_menu - 1].xpos + strlen(new_menus[cur_menu - 1].text) + 1;
 												new_menus[cur_menu].items = &new_items[cur_item];
 											}
 											else error_in_menu_configuration(line, "menu name has to end with quotes.");
@@ -867,7 +906,7 @@ void get_menu_configuration(char *menu_conf_name) {
 										else error_in_menu_configuration(line, "menu name has to start with quotes.");
 									}
 								}
-								else if (cur_menu >= 0) error_in_menu_configuration(line-1, "no items specified for this menu.");
+								else if (cur_menu >= 0) error_in_menu_configuration(line - 1, "no items specified for this menu.");
 
 							}
 							else if (!cmdcmp(ITEM_KEYWORD, p)) {
@@ -885,10 +924,10 @@ void get_menu_configuration(char *menu_conf_name) {
 													if ((new_menus[cur_menu].width = strlen(new_items[cur_item].text)) == 0)
 														error_in_menu_configuration(line, "menu item name width has to be greater than zero.");
 												}
-												while (isspace((unsigned char)*p)) p++;
+												while (isasciispace(*p)) p++;
 												if (*p) {
 													new_items[cur_item].command_line = p;
-													new_menus[cur_menu].item_num = num_items_in_menu+1;
+													new_menus[cur_menu].item_num = num_items_in_menu + 1;
 												}
 												else error_in_menu_configuration(line, "no command specified.");
 											}
@@ -904,18 +943,18 @@ void get_menu_configuration(char *menu_conf_name) {
 
 						}
 						line++;
-						p += strlen(p)+1;
+						p += strlen(p) + 1;
 					}
 
 					if (pass == 0) {
-						if (!num_items_in_menu) error_in_menu_configuration(line-1, "no items specified for this menu.");
+						if (!num_items_in_menu) error_in_menu_configuration(line - 1, "no items specified for this menu.");
 						if (cur_menu == -1 || cur_item == 0) error_in_menu_configuration(line, "no menus or items specified.");
 
-						if (!(new_menus = calloc(cur_menu+1, sizeof(menu))) || !(new_items = calloc(cur_item, sizeof(menu_item))))
+						if (!(new_menus = calloc(cur_menu + 1, sizeof(menu))) || !(new_items = calloc(cur_item, sizeof(menu_item))))
 							error_in_menu_configuration(line, "not enough memory.");
 					}
 					else {
-						menu_num = cur_menu+1;
+						menu_num = cur_menu + 1;
 						menus = new_menus;
 					}
 				}
@@ -927,22 +966,22 @@ void get_menu_configuration(char *menu_conf_name) {
 }
 
 
-static void error_in_key_bindings(int line, char *s) {
+static void error_in_key_bindings(const int line, const char * const s) {
 
 	fprintf(stderr, "Error in key bindings file at line %d: %s\n", line, s);
 	exit(0);
 }
 
-void get_key_bindings(char *key_bindings_name) {
+void get_key_bindings(const char * key_bindings_name) {
 	char *prefs_dir, *key_bindings;
 	char_stream *cs;
-	char *p;
+	unsigned char *p;
 	int c, line;
 
 	if (!key_bindings_name) key_bindings_name = KEY_BINDINGS_NAME;
 
 	if (prefs_dir = exists_prefs_dir()) {
-		if (key_bindings = malloc(strlen(prefs_dir)+strlen(key_bindings_name)+1)) {
+		if (key_bindings = malloc(strlen(prefs_dir) + strlen(key_bindings_name) + 1)) {
 			strcat(strcpy(key_bindings, prefs_dir), key_bindings_name);
 
 			if ((cs = load_stream(NULL, key_bindings_name, FALSE)) || (cs = load_stream(NULL, key_bindings, FALSE))) {
@@ -950,16 +989,16 @@ void get_key_bindings(char *key_bindings_name) {
 				p = cs->stream;
 				line = 1;
 
-				while(p-cs->stream<cs->len) {
+				while(p - cs->stream < cs->len) {
 					if (*p && !cmdcmp(KEY_KEYWORD, p)) {
-						while(*p && !isspace((unsigned char)*p)) p++;
+						while(*p && !isasciispace(*p)) p++;
 
 						if (sscanf(p, "%x %*s", &c) == 1) {
-							if (c>=0 && c<NUM_KEYS) {
+							if (c >= 0 && c < NUM_KEYS) {
 								if (c != 27 && c != 13) {
-									while(isspace((unsigned char)*p)) p++;
-									while(*p && !isspace((unsigned char)*p)) p++;
-									while(isspace((unsigned char)*p)) p++;
+									while(isasciispace(*p)) p++;
+									while(*p && !isasciispace(*p)) p++;
+									while(isasciispace(*p)) p++;
 									if (*p) key_binding[c] = p;
 									else error_in_key_bindings(line, "no command specified.");
 								}
@@ -971,7 +1010,7 @@ void get_key_bindings(char *key_bindings_name) {
 					}
 
 					line++;
-					p += strlen(p)+1;
+					p += strlen(p) + 1;
 				}
 			}
 
