@@ -1,7 +1,7 @@
 /* Extern's for names and abbreviations of all the commands.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2001 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2004 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -369,6 +369,15 @@ extern const char UNDO_ABBREV[];
 extern const char UNLOADMACROS_NAME[];
 extern const char UNLOADMACROS_ABBREV[];
 
+extern const char UTF8_NAME[];
+extern const char UTF8_ABBREV[];
+
+extern const char UTF8AUTO_NAME[];
+extern const char UTF8AUTO_ABBREV[];
+
+extern const char UTF8IO_NAME[];
+extern const char UTF8IO_ABBREV[];
+
 extern const char VERBOSEMACROS_NAME[];
 extern const char VERBOSEMACROS_ABBREV[];
 
