@@ -1,7 +1,7 @@
 /* Navigation functions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2001 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2004 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -55,8 +55,8 @@ familiar with this diagram.
     cur_y |         |                         |          |  |         |       |
       |   |         |                         |          |  |         |       |
       |   |         |                         |          |  |         |       |
-     ---  |         |         @ <-Cursor      |          | ---        |      lines   
-          |         |                         |          |          line_num  |
+     ---  |         |         @ <-Cursor      |          | ---        |      ne_lines   
+          |         |                         |          |          num_lines |
           |         |                         |          |            |       |
           |         |                         |          |            |       |
           |         |                         |          |            |       |
@@ -66,81 +66,90 @@ familiar with this diagram.
           |                                              |            |
           +----------------------------------------------+          -----
 
-                    |<---- columns ---------->|
+                    |<---- ne_columns ------->|
          
 ************************************************************************************/
 
           
-/* This functions "resyncs" cur_pos (the current character the cursor is on)
-with cur_x and win_x. It has to take into account the TAB expansion, and can
-cause left/right movement in order to properly land on a real character.
-x is the offset from the beginning of the line after TAB expansion. resync_pos()
-assumes that tab_size < columns/2. Note that this function has to be called
-whenever the cursor is moved to a different line, keeping the x position
-constant. The only way of avoiding this problem is not supporting TABs,
-which is of course unacceptable. Note that if x_wanted is TRUE, then the
-wanted_x position is used rather tham cur_x+win_x. */
+/* "Resyncs" cur_pos (the current character the cursor is on) with cur_x and
+	win_x. It has to take into account the TAB expansion, and can cause
+	left/right movement in order to properly land on a real character.  x is the
+	offset from the beginning of the line after TAB expansion. resync_pos()
+	assumes that tab_size < columns/2. Note that this function has to be called
+	whenever the cursor is moved to a different line, keeping the x position
+	constant. The only way of avoiding this problem is not supporting TABs,
+	which is of course unacceptable. Note that if x_wanted is TRUE, then the
+	wanted_x position is used rather tham cur_x+win_x. */
 
-static void resync_pos(buffer *b) {
+static void resync_pos(buffer * const b) {
 
 	line_desc *ld = b->cur_line_desc;
 
-	int i, len, last_tab_skip, x = b->win_x+b->cur_x;
+	int pos, width, last_char_width, x = b->win_x + b->cur_x;
 
 	if (b->x_wanted) x = b->wanted_x;
 
-	assert(b->opt.tab_size < ne_columns/2);
+	assert(b->opt.tab_size < ne_columns / 2);
 
 	if (x == 0) {
 		b->cur_pos = 0;
 		return;
 	}
 
-	for(i=len=0; i<ld->line_len; i++) {
+	for(pos = width = 0; pos < ld->line_len; pos = next_pos(ld->line, pos, b->encoding)) {
 
-		if (ld->line[i] != '\t') len++;
+		if (ld->line[pos] != '\t') width += (last_char_width = get_char_width(&ld->line[pos], b->encoding));
+		else width += (last_char_width = b->opt.tab_size - width % b->opt.tab_size);
 
-		else len += (last_tab_skip = b->opt.tab_size - len%b->opt.tab_size);
-
-		if (len == x) {
-			b->cur_pos = i+1;
+		if (width == x) {
+			b->cur_pos = pos + (b->encoding == ENC_UTF8 ? utf8len(ld->line[pos]) : 1);
 			if (b->x_wanted) {
 
 				b->x_wanted = 0;
 
-				if (x-b->win_x < ne_columns) b->cur_x = x-b->win_x;
+				if (x - b->win_x < ne_columns) b->cur_x = x - b->win_x;
 				else {
-					b->win_x = x-ne_columns;
+					b->win_x = x - ne_columns;
 					b->win_x += b->opt.tab_size - b->win_x % b->opt.tab_size;
-					b->cur_x = x-b->win_x;
-					if ( b == cur_buffer) update_window(b);
+					b->cur_x = x - b->win_x;
+					if (b == cur_buffer) update_window(b);
 				}
 			}
 			return;
 		}
 
-		if (len > x) {
+		if (width > x) {
 
-			b->cur_pos = i;
+			b->cur_pos = pos;
 
-			len -= last_tab_skip;
+			width -= last_char_width;
 			b->x_wanted = 1;
 			b->wanted_x = x;
 
-			if (len-b->win_x < ne_columns) b->cur_x = len-b->win_x;
+			if (width - b->win_x < 0) {
+				/* We are on a character which is only partially on the screen
+					(more precisely, its right margin is not). We shift the screen
+					to the left. */
+				assert(b->win_x > 0);
+				b->win_x = max(0, width - ne_columns);
+				b->win_x -= b->win_x % b->opt.tab_size;
+				b->cur_x = width - b->win_x;
+				if (b == cur_buffer) update_window(b);
+			}
+			else if (width - b->win_x < ne_columns) b->cur_x = width - b->win_x;
 			else {
-				b->win_x = len-ne_columns;
+				b->win_x = width - ne_columns;
 				b->win_x += b->opt.tab_size - b->win_x % b->opt.tab_size;
-				b->cur_x = len-b->win_x;
-				if ( b == cur_buffer) update_window(b);
+				b->cur_x = width - b->win_x;
+				if (b == cur_buffer) update_window(b);
 			}
 			return;
 		}
 	}
 
 	if (b->opt.free_form) {
-		b->cur_pos = ld->line_len + x-len;
-		b->cur_x = x-b->win_x;
+		b->cur_pos = ld->line_len + x - width;
+		b->cur_x = x - b->win_x;
 		b->x_wanted = 0;
 	}
 	else {
@@ -152,54 +161,54 @@ static void resync_pos(buffer *b) {
 
 
 
-int line_up(buffer *b) {
+int line_up(buffer * const b) {
 
 	b->y_wanted = 0;
-	if (b->cur_y>0) {
+	if (b->cur_y > 0) {
 		b->cur_y--;
 		b->cur_line--;
 		b->cur_line_desc = (line_desc *)b->cur_line_desc->ld_node.prev;
 		resync_pos(b);
-		return(OK);
+		return OK;
 	}
 	else {
-		if (b->win_y>0) {
+		if (b->win_y > 0) {
 			b->win_y--;
 			b->cur_line--;
 			b->cur_line_desc = (line_desc *)b->cur_line_desc->ld_node.prev;
 			b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.prev;
 			if (b == cur_buffer) scroll_window(b, 0, 1);
 			resync_pos(b);
-			return(OK);
+			return OK;
 		}
 	}
-	return(ERROR);
+	return ERROR;
 }
 
 
 
-int line_down(buffer *b) {
+int line_down(buffer * const b) {
 
 	b->y_wanted = 0;
-	if (b->cur_y<ne_lines-2 && b->cur_line<b->line_num-1) {
+	if (b->cur_y < ne_lines - 2 && b->cur_line < b->num_lines - 1) {
 		b->cur_y++;
 		b->cur_line++;
 		b->cur_line_desc = (line_desc *)b->cur_line_desc->ld_node.next;
 		resync_pos(b);
-		return(OK);
+		return OK;
 	}
 	else {
-		if (b->win_y<b->line_num-ne_lines+1) {
+		if (b->win_y < b->num_lines - ne_lines + 1) {
 			b->win_y++;
 			b->cur_line++;
 			b->cur_line_desc = (line_desc *)b->cur_line_desc->ld_node.next;
 			b->top_line_desc = (line_desc *)b->top_line_desc->ld_node.next;
 			if (b == cur_buffer) scroll_window(b, 0, -1);
 			resync_pos(b);
-			return(OK);
+			return OK;
 		}
 	}
-	return(ERROR);
+	return ERROR;
 }
 
 /* This has to be done whenever we switch to a different buffer
@@ -207,11 +216,11 @@ int line_down(buffer *b) {
  * were here.
  */
  
-void keep_cursor_on_screen( buffer *b )
+void keep_cursor_on_screen(buffer * const b)
   {
-    if ( b->cur_y > ne_lines-2 )
+    if ( b->cur_y > ne_lines - 2 )
         {
-          while( b->cur_y > ne_lines-2 )
+          while( b->cur_y > ne_lines - 2 )
             {
               b->cur_y--;
               b->win_y++;
@@ -230,97 +239,101 @@ void keep_cursor_on_screen( buffer *b )
 
 
 
-/* This function moves win_x of n bytes to the left (n *has* to be a
-multiple of the current TAB size). It is used by char_left(). cur_x
-is moved, too. */
+/* Moves win_x of n bytes to the left (n *has* to be a multiple of the current
+	TAB size). It is used by char_left(). cur_x is moved, too. */
 
-static void block_left(buffer *b, int n) {
+static void block_left(buffer * const b, const int n) {
 
 	int t = b->win_x;
 
 	assert(n <= ne_columns);
-	assert(!(n%b->opt.tab_size));
+	assert(!(n % b->opt.tab_size));
 
 	if ((b->win_x -= n) < 0) b->win_x = 0;
-	b->cur_x += t-b->win_x;
-	if ( b == cur_buffer) update_window(b);
+	b->cur_x += t - b->win_x;
+	if (b == cur_buffer) update_window(b);
 }
 
 
 
-int char_left(buffer *b) {
+int char_left(buffer * const b) {
 
 	line_desc *ld = b->cur_line_desc;
 
 	assert(ld != NULL);
-	assert_line_desc(ld);
+	assert_line_desc(ld, b->encoding);
 
 	b->x_wanted = 0;
 	b->y_wanted = 0;
 
 	if (b->cur_pos > 0) {
 
-		int disp = 1;
+		int disp = ld->line && b->cur_pos <= ld->line_len ? get_char_width(&ld->line[prev_pos(ld->line, b->cur_pos, b->encoding)], b->encoding) : 1;
 
-		if (b->cur_pos <= ld->line_len && ld->line[b->cur_pos-1] == 0x9)
-			disp = b->opt.tab_size - calc_len(ld, b->cur_pos-1, b->opt.tab_size)%b->opt.tab_size;
+		if (b->cur_pos <= ld->line_len && ld->line[b->cur_pos - 1] == '\t')
+			disp = b->opt.tab_size - calc_width(ld, b->cur_pos - 1, b->opt.tab_size, b->encoding) % b->opt.tab_size;
 
-		if (b->cur_x < disp) block_left(b, b->opt.tab_size*2);
+		if (b->cur_x < disp) block_left(b, b->opt.tab_size * 2);
 		b->cur_x -= disp;
-		b->cur_pos--;
-		return(OK);
+
+		/* If the buffer is UTF-8 encoded, we move back until we find a sequence initiator. */
+		b->cur_pos = b->cur_pos > ld->line_len ? b->cur_pos - 1 : prev_pos(ld->line, b->cur_pos, b->encoding);
+		return OK;
 	}
-	else if (b->cur_line>0) {
+	else if (b->cur_line > 0) {
 		line_up(b);
 		move_to_eol(b);
-		return(OK);
+		return OK;
 	}
-	return(ERROR);
+	return ERROR;
 }
 
 
 /* Same as block_left(), but to the right. */
 
-static void block_right(buffer *b, int n) {
+static void block_right(buffer * const b, const int n) {
 
 	assert(n <= ne_columns);
-	assert(!(n%b->opt.tab_size));
+	assert(!(n % b->opt.tab_size));
 
 	b->win_x += n;
 	b->cur_x -= n;
-	if ( b == cur_buffer) update_window(b);
+	if (b == cur_buffer) update_window(b);
 }
 
-int char_right(buffer *b) {
+int char_right(buffer * const b) {
 
-	line_desc *ld = b->cur_line_desc;
-	int disp = 1;
+	const line_desc * const ld = b->cur_line_desc;
+	int disp = ld->line && b->cur_pos < ld->line_len ? get_char_width(&ld->line[b->cur_pos], b->encoding) : 1;
 
 	assert(ld != NULL);
-	assert_line_desc(ld);
+	assert_line_desc(ld, b->encoding);
 
-	if (ld->line && b->cur_pos < ld->line_len && ld->line[b->cur_pos] == 0x9)
-		disp = b->opt.tab_size - calc_len(ld, b->cur_pos, b->opt.tab_size)%b->opt.tab_size;
+	if (ld->line && b->cur_pos < ld->line_len && ld->line[b->cur_pos] == '\t')
+		disp = b->opt.tab_size - calc_width(ld, b->cur_pos, b->opt.tab_size, b->encoding) % b->opt.tab_size;
 
 	b->x_wanted = 0;
 	b->y_wanted = 0;
 
 	if (b->cur_pos == ld->line_len && !b->opt.free_form) {
-		if (!ld->ld_node.next->next) return(ERROR);
+		if (!ld->ld_node.next->next) return ERROR;
 		move_to_sol(b);
 		line_down(b);
-		return(OK);
+		return OK;
 	}
 
-	if (b->cur_x+disp >= ne_columns) block_right(b, b->opt.tab_size*2);
 	b->cur_x += disp;
-	b->cur_pos++;
-	return(OK);
+	b->cur_pos = b->cur_pos >= ld->line_len ? b->cur_pos + 1 : next_pos(ld->line, b->cur_pos, b->encoding);
+
+	/* If the current x position would be beyond the right screen margin, or if the same happens
+		for the character we are currently over, we shift the screen to the right. */
+	if (b->cur_x >= ne_columns || ld->line && b->cur_pos < ld->line_len && b->cur_x + get_char_width(&ld->line[b->cur_pos], b->encoding) > ne_columns) block_right(b, b->opt.tab_size * 2);
+	return OK;
 }
 
 
 
-int prev_page(buffer *b) {
+int prev_page(buffer * const b) {
 
 	int i;
 	line_desc *ld_top, *ld_cur;
@@ -332,17 +345,17 @@ int prev_page(buffer *b) {
 		b->cur_y = 0;
 		b->cur_line_desc = b->top_line_desc;
 		resync_pos(b);
-		return(OK);
+		return OK;
 	}
 
-	if (b->win_y == 0) return(ERROR);
+	if (b->win_y == 0) return ERROR;
 
-	if ((b->win_y -= ne_lines-2)<0) b->win_y = 0;
+	if ((b->win_y -= ne_lines - 2)<0) b->win_y = 0;
 
 	ld_top = b->top_line_desc;
 	ld_cur = b->cur_line_desc;
 
-	for(i=0; i<ne_lines-2 && ld_top->ld_node.prev->prev; i++) {
+	for(i = 0; i < ne_lines - 2 && ld_top->ld_node.prev->prev; i++) {
 		ld_top = (line_desc *)ld_top->ld_node.prev;
 		ld_cur = (line_desc *)ld_cur->ld_node.prev;
 		b->cur_line--;
@@ -351,45 +364,45 @@ int prev_page(buffer *b) {
 	b->top_line_desc = ld_top;
 	b->cur_line_desc = ld_cur;
 
-	if ( b == cur_buffer) update_window(b);
+	if (b == cur_buffer) update_window(b);
 	resync_pos(b);
-	return(ERROR);
+	return ERROR;
 }
 
 
 
-int next_page(buffer *b) {
+int next_page(buffer * const b) {
 	int i, disp;
 	line_desc *ld_top, *ld_cur;
 
 	b->y_wanted = 0;
 
-	if (b->cur_y < ne_lines-2) {
-		if (b->win_y >= b->line_num - (ne_lines-1)) {
+	if (b->cur_y < ne_lines - 2) {
+		if (b->win_y >= b->num_lines - (ne_lines - 1)) {
 			ld_cur = b->top_line_desc;
-			for(i=0; i<ne_lines-2 && ld_cur->ld_node.next->next; i++)
+			for(i = 0; i < ne_lines - 2 && ld_cur->ld_node.next->next; i++)
 			   ld_cur = (line_desc *)ld_cur->ld_node.next;
 			b->cur_line += (i - b->cur_y);
 			b->cur_y = i;
 		}
 		else {
 			b->cur_line += (ne_lines - 2 - b->cur_y);
-			b->cur_y = ne_lines-2;
+			b->cur_y = ne_lines - 2;
 			ld_cur = b->top_line_desc;
-			for(i=0; i<ne_lines-2; i++)
+			for(i = 0; i < ne_lines - 2; i++)
 			  ld_cur = (line_desc *)ld_cur->ld_node.next;
 		}
 		b->cur_line_desc = ld_cur;
-		if ( b == cur_buffer) resync_pos(b);
-		return(OK);
+		resync_pos(b);
+		return OK;
 	}
 
-	if (b->win_y >= b->line_num-(ne_lines-1)) return(ERROR);
+	if (b->win_y >= b->num_lines - (ne_lines - 1)) return ERROR;
 
-	disp = ne_lines-2;
+	disp = ne_lines - 2;
 
-	if (b->win_y + disp > b->line_num-(ne_lines-1))
-		disp = b->line_num-(ne_lines-1) - b->win_y;
+	if (b->win_y + disp > b->num_lines - (ne_lines - 1))
+		disp = b->num_lines - (ne_lines - 1) - b->win_y;
 
 	b->win_y += disp;
 	b->cur_line += disp;
@@ -397,7 +410,7 @@ int next_page(buffer *b) {
 	ld_top = b->top_line_desc;
 	ld_cur = b->cur_line_desc;
 
-	for(i=0; i<disp && ld_top->ld_node.next->next; i++) {
+	for(i = 0; i < disp && ld_top->ld_node.next->next; i++) {
 		ld_top = (line_desc *)ld_top->ld_node.next;
 		ld_cur = (line_desc *)ld_cur->ld_node.next;
 	}
@@ -405,19 +418,19 @@ int next_page(buffer *b) {
 	b->top_line_desc = ld_top;
 	b->cur_line_desc = ld_cur;
 
-	if ( b == cur_buffer) update_window(b);
+	if (b == cur_buffer) update_window(b);
 	resync_pos(b);
-	return(OK);
+	return OK;
 }
 
-int page_up(buffer *b) {
+int page_up(buffer * const b) {
 
 	int i, disp;
 
 	disp = ne_lines - 2;
 
 	/* Already on the top line? */
-	if ( b->cur_line == 0 ) return(OK);
+	if ( b->cur_line == 0 ) return OK;
 	
 	if (  !b->y_wanted )
 		{
@@ -426,13 +439,13 @@ int page_up(buffer *b) {
 			b->wanted_cur_y = b->cur_y;
 	   }
 
-	for (i=0; i<disp; i++)
+	for (i = 0; i < disp; i++)
    	{
 			b->wanted_y--;                        /* We want to move up */
 			
 			/* Can we move up? */
 			if (     b->wanted_y >= 0             /* We aren't yet off the top    */
-			     &&  b->wanted_y < b->line_num-1) /* we aren't still past the end */
+			     &&  b->wanted_y < b->num_lines - 1) /* we aren't still past the end */
 				{
 					b->cur_line_desc = (line_desc *)b->cur_line_desc->ld_node.prev;
 					b->cur_line--;
@@ -450,19 +463,19 @@ int page_up(buffer *b) {
 	b->cur_y = b->cur_line - b->win_y;
 
 	keep_cursor_on_screen( b );
-	if ( b == cur_buffer) update_window(b);
+	if (b == cur_buffer) update_window(b);
 	resync_pos(b);
-	return(OK);
+	return OK;
 }
 
-int page_down(buffer *b) {
+int page_down(buffer * const b) {
 
 	int i, disp, shift_view;
 
 	disp = ne_lines - 2;
 
 	/* Already on the bottom line? */
-	if ( b->cur_line == b->line_num-1 ) return(OK);
+	if ( b->cur_line == b->num_lines - 1 ) return OK;
 	
 	if ( !b->y_wanted )
 		{
@@ -471,15 +484,15 @@ int page_down(buffer *b) {
 			b->wanted_cur_y = b->cur_y;
 	   }
 
-   shift_view = ( b->win_y+disp < b->line_num ); /* can't already see the last line */
+   shift_view = ( b->win_y + disp < b->num_lines ); /* can't already see the last line */
    
-	for (i=0; i<disp; i++)
+	for (i = 0; i < disp; i++)
    	{
 			b->wanted_y++;                        /* We want to move down */
 			
 			/* Can we move down? */
 			if (     b->wanted_y > 0              /* We aren't still above the top  */
-			     &&  b->wanted_y < b->line_num )  /* we aren't yet to the end       */
+			     &&  b->wanted_y < b->num_lines )  /* we aren't yet to the end       */
 				{
 					b->cur_line_desc = (line_desc *)b->cur_line_desc->ld_node.next;
 					b->cur_line++;
@@ -497,12 +510,12 @@ int page_down(buffer *b) {
 	b->cur_y = b->cur_line - b->win_y;
 
 	keep_cursor_on_screen( b );
-	if ( b == cur_buffer) update_window(b);
+	if (b == cur_buffer) update_window(b);
 	resync_pos(b);
-	return(OK);
+	return OK;
 }
 
-int move_tos(buffer *b) {
+int move_tos(buffer * const b) {
 
 	b->y_wanted = 0;
 
@@ -512,43 +525,43 @@ int move_tos(buffer *b) {
 		b->cur_line_desc = b->top_line_desc;
 		resync_pos(b);
 	}
-	return(OK);
+	return OK;
 }
 
-int move_bos(buffer *b) {
+int move_bos(buffer * const b) {
 	int i;
 	line_desc *ld_cur;
 
 	b->y_wanted = 0;
 
-	if (b->cur_y < ne_lines-2) {
-		if (b->win_y >= b->line_num - (ne_lines-1)) {
+	if (b->cur_y < ne_lines - 2) {
+		if (b->win_y >= b->num_lines - (ne_lines - 1)) {
 			ld_cur = b->top_line_desc;
-			for(i=0; i<ne_lines-2  &&   ld_cur->ld_node.next->next; i++)
+			for(i = 0; i < ne_lines - 2  &&   ld_cur->ld_node.next->next; i++)
 			   ld_cur = (line_desc *)ld_cur->ld_node.next;
 			b->cur_line += (i - b->cur_y);
 			b->cur_y = i;
 		}
 		else {
 			b->cur_line += (ne_lines - 2 - b->cur_y);
-			b->cur_y = ne_lines-2;
+			b->cur_y = ne_lines - 2;
 			ld_cur = b->top_line_desc;
-			for(i=0; i<ne_lines-2; i++)
+			for(i = 0; i < ne_lines - 2; i++)
 			  ld_cur = (line_desc *)ld_cur->ld_node.next;
 		}
 		b->cur_line_desc = ld_cur;
 		resync_pos(b);
 	}
-	return(OK);
+	return OK;
 }
 
 /* adjust_view() never moves the cursor. It is only concerned with shifting
    win_x, cur_x, win_y and cur_y -- the variables which control which part
    of the file is visible in the terminal window. */
 
-int adjust_view(buffer *b, char *p)
+int adjust_view(buffer * const b, const unsigned char *p)
 	{
-		int i, disp, rc=OK;
+		int i, disp, rc = OK;
 
 		b->y_wanted = 0;
 
@@ -568,13 +581,13 @@ int adjust_view(buffer *b, char *p)
 						case 'c' :
 						case 'C' :
 							/* Shift the view so that the current line is displayed at the center. */
-							disp = (ne_lines-2)/2 - b->cur_y;
+							disp = (ne_lines - 2) / 2 - b->cur_y;
 							break;
 
 						case 'b' :
 						case 'B' :
 							/* Shift the view so that the current line is displayed at the bottom. */
-							disp = (ne_lines-2)   - b->cur_y;
+							disp = (ne_lines - 2) - b->cur_y;
 							break;
 
 						case 'l' :
@@ -599,7 +612,7 @@ int adjust_view(buffer *b, char *p)
 							/* Since we now know that the cursor is left of center, we can start
 							   to shift the view right until the cursor is centered or until we
 							   run out of text to shift right. */
-							while ( b->cur_x < (ne_columns/2) - (ne_columns/2) % b->opt.tab_size &&
+							while ( b->cur_x < (ne_columns / 2) - (ne_columns / 2) % b->opt.tab_size &&
 							        b->win_x >= b->opt.tab_size                      )
 								{
 									b->win_x -= b->opt.tab_size;
@@ -626,7 +639,7 @@ int adjust_view(buffer *b, char *p)
 					}
 				if ( disp > 0 )
 						{
-							for(i=0; i<disp && b->top_line_desc->ld_node.prev->prev; i++)
+							for(i = 0; i < disp && b->top_line_desc->ld_node.prev->prev; i++)
 								{
 									b->win_y--;
 									b->cur_y++;
@@ -635,7 +648,7 @@ int adjust_view(buffer *b, char *p)
 						}
 					else if ( disp < 0 )
 						{
-							for(i=0; i>disp && b->top_line_desc->ld_node.next->next; i--)
+							for(i = 0; i > disp && b->top_line_desc->ld_node.next->next; i--)
 								{
 									b->win_y++;
 									b->cur_y--;
@@ -644,75 +657,77 @@ int adjust_view(buffer *b, char *p)
 						}
 				p++;
 			}
-		if ( b == cur_buffer) update_window(b);
+		if (b == cur_buffer) update_window(b);
 		resync_pos(b);
 		return rc;
 	}
 
-void goto_line(buffer *b, int n) {
+
+
+void goto_line(buffer * const b, const int n) {
 	int i;
 	line_desc *ld;
 
 	b->y_wanted = 0;
 
-	if (n >= b->line_num || n == b->cur_line) return;
+	if (n >= b->num_lines || n == b->cur_line) return;
 
-	if (n >= b->win_y && n < b->win_y+ne_lines-1) {
-		b->cur_y = n-b->win_y;
+	if (n >= b->win_y && n < b->win_y + ne_lines - 1) {
+		b->cur_y = n - b->win_y;
 		b->cur_line = n;
 		ld = b->top_line_desc;
-		for(i=0; i<b->cur_y; i++) ld = (line_desc *)ld->ld_node.next;
+		for(i = 0; i < b->cur_y; i++) ld = (line_desc *)ld->ld_node.next;
 		b->cur_line_desc = ld;
 		resync_pos(b);
 		return;
 	}
 
-	b->win_y = n - (ne_lines-1)/2;
+	b->win_y = n - (ne_lines - 1) / 2;
 
-	if (b->win_y > b->line_num-(ne_lines-1)) b->win_y = b->line_num-(ne_lines-1);
+	if (b->win_y > b->num_lines - (ne_lines - 1)) b->win_y = b->num_lines - (ne_lines - 1);
 	if (b->win_y < 0) b->win_y = 0;
 
 	b->cur_y = n - b->win_y;
 	b->cur_line = n;
 
-	if (n<b->line_num/2) {
+	if (n < b->num_lines / 2) {
 		ld = (line_desc *)b->line_desc_list.head;
-		for(i=0; i<b->win_y; i++) ld = (line_desc *)ld->ld_node.next;
+		for(i = 0; i < b->win_y; i++) ld = (line_desc *)ld->ld_node.next;
 	}
 	else {
 		ld = (line_desc *)b->line_desc_list.tail_pred;
-		for(i=b->line_num-1; i>b->win_y; i--) ld = (line_desc *)ld->ld_node.prev;
+		for(i = b->num_lines - 1; i > b->win_y; i--) ld = (line_desc *)ld->ld_node.prev;
 	}
 
 	b->top_line_desc = ld;
-	for(i=0; i<b->cur_y; i++) ld = (line_desc *)ld->ld_node.next;
+	for(i = 0; i < b->cur_y; i++) ld = (line_desc *)ld->ld_node.next;
 	b->cur_line_desc = ld;
 
-	if ( b == cur_buffer) update_window(b);
+	if (b == cur_buffer) update_window(b);
 	resync_pos(b);
 }
 
 
 
-void goto_column(buffer *b, int n) {
+void goto_column(buffer * const b, const int n) {
 
 	b->x_wanted = 0;
 	b->y_wanted = 0;
 
-	if (n == b->win_x+b->cur_x) return;
+	if (n == b->win_x + b->cur_x) return;
 
-	if (n >= b->win_x && n < b->win_x+ne_columns) {
-		b->cur_x = n-b->win_x;
+	if (n >= b->win_x && n < b->win_x + ne_columns) {
+		b->cur_x = n - b->win_x;
 		resync_pos(b);
 		return;
 	}
 
-	if ((b->win_x = n - ne_columns/2)<0) b->win_x = 0;
-	b->win_x -= b->win_x%b->opt.tab_size;
-	b->cur_x = n-b->win_x;
+	if ((b->win_x = n - ne_columns / 2)<0) b->win_x = 0;
+	b->win_x -= b->win_x % b->opt.tab_size;
+	b->cur_x = n - b->win_x;
 
 	resync_pos(b);
-	if ( b == cur_buffer) update_window(b);
+	if (b == cur_buffer) update_window(b);
 }
 
 
@@ -720,14 +735,14 @@ void goto_column(buffer *b, int n) {
 /* This is like a goto_column(), but you specify a position (i.e.,
 a character offset) instead. */
 
-void goto_pos(buffer *b, int pos) {
+void goto_pos(buffer * const b, const int pos) {
 
-	goto_column(b, calc_len(b->cur_line_desc, pos, b->opt.tab_size));
+	goto_column(b, calc_width(b->cur_line_desc, pos, b->opt.tab_size, b->encoding));
 }
 
 
 
-void move_to_sol(buffer *b) {
+void move_to_sol(buffer * const b) {
 
 	int t;
 
@@ -739,11 +754,11 @@ void move_to_sol(buffer *b) {
 	b->cur_x =
 	b->cur_pos = 0;
 
-	if (t &&  b == cur_buffer ) update_window(b);
+	if (t &&  b == cur_buffer) update_window(b);
 }
 
 
-void move_to_eol(buffer *b) {
+void move_to_eol(buffer * const b) {
 	int i, len, total_len;
 	line_desc *ld = b->cur_line_desc;
 
@@ -758,21 +773,21 @@ void move_to_eol(buffer *b) {
 		return;
 	}
 
-	total_len = calc_len(ld, ld->line_len, b->opt.tab_size);
+	total_len = calc_width(ld, ld->line_len, b->opt.tab_size, b->encoding);
 
-	if (total_len >= b->win_x && total_len < b->win_x+ne_columns) {
+	if (total_len >= b->win_x && total_len < b->win_x + ne_columns) {
 		b->cur_x = total_len - b->win_x;
 		b->cur_pos = ld->line_len;
 		return;
 	}
 
-	for(len=0, i=0; i<ld->line_len; i++)  {
+	for(len = 0, i = 0; i < ld->line_len; i++)  {
 		if (ld->line[i] != '\t') len++;
-		else len += b->opt.tab_size - len%b->opt.tab_size;
+		else len += b->opt.tab_size - len % b->opt.tab_size;
 
-		if (total_len-len < ne_columns - b->opt.tab_size) {
+		if (total_len - len < ne_columns - b->opt.tab_size) {
 			int t = b->win_x;
-			b->win_x = len-len%b->opt.tab_size;
+			b->win_x = len - len % b->opt.tab_size;
 			b->cur_x = total_len - b->win_x;
 			b->cur_pos = ld->line_len;
 			if (t != b->win_x) update_window(b);
@@ -785,10 +800,10 @@ void move_to_eol(buffer *b) {
 
 
 
-/* This function sets the variables like a move_to_sof(), but does not
-perform any update. This is required in several places. */
+/* Sets the variables like a move_to_sof(), but does not perform any
+	update. This is required in several places. */
 
-void reset_position_to_sof(buffer *b) {
+void reset_position_to_sof(buffer * const b) {
 	b->x_wanted =
 	b->y_wanted =
 	b->win_x =
@@ -802,19 +817,19 @@ void reset_position_to_sof(buffer *b) {
 
 
 
-void move_to_sof(buffer *b) {
+void move_to_sof(buffer * const b) {
 
 	int old_win_x = b->win_x, old_win_y = b->win_y;
 
 	reset_position_to_sof(b);
 
 	if (old_win_x != b->win_x || old_win_y != b->win_y)
-		if ( b == cur_buffer) update_window(b);
+		if (b == cur_buffer) update_window(b);
 }
 
 
 
-void move_to_bof(buffer *b) {
+void move_to_bof(buffer * const b) {
 	int i;
 	int old_win_x = b->win_x, old_win_y = b->win_y;
 	line_desc *ld = (line_desc *)b->line_desc_list.tail_pred;
@@ -822,16 +837,16 @@ void move_to_bof(buffer *b) {
 	b->x_wanted = 0;
 	b->y_wanted = 0;
 
-	for(i=0; i<ne_lines-2 && ld->ld_node.prev->prev; i++) ld = (line_desc *)ld->ld_node.prev;
+	for(i = 0; i < ne_lines - 2 && ld->ld_node.prev->prev; i++) ld = (line_desc *)ld->ld_node.prev;
 
-	b->cur_line = b->line_num-1;
+	b->cur_line = b->num_lines - 1;
 
 	if (!ld->ld_node.prev->prev) {
 		b->win_x =
 		b->win_y =
 		b->cur_x =
 		b->cur_pos = 0;
-		b->cur_y = b->line_num-1;
+		b->cur_y = b->num_lines - 1;
 		b->top_line_desc = (line_desc *)b->line_desc_list.head;
 		b->cur_line_desc = (line_desc *)b->line_desc_list.tail_pred;
 	}
@@ -839,21 +854,22 @@ void move_to_bof(buffer *b) {
 		b->win_x =
 		b->cur_x =
 		b->cur_pos = 0;
-		b->win_y = b->line_num - (ne_lines-1);
-		b->cur_y = ne_lines-2;
+		b->win_y = b->num_lines - (ne_lines - 1);
+		b->cur_y = ne_lines - 2;
 		b->top_line_desc = ld;
 		b->cur_line_desc = (line_desc *)b->line_desc_list.tail_pred;
 	}
 
 	if (old_win_x != b->win_x || old_win_y != b->win_y)
-		if ( b == cur_buffer) update_window(b);
+		if (b == cur_buffer) update_window(b);
 }
 
 
 
-void toggle_sof_eof(buffer *b) {
+void toggle_sof_eof(buffer * const b) {
 
 	if (b->cur_line == 0 && b->cur_pos == 0) {
+		delay_update();
 		move_to_bof(b);
 		move_to_eol(b);
 	}
@@ -862,7 +878,7 @@ void toggle_sof_eof(buffer *b) {
 
 
 
-void toggle_sol_eol(buffer *b) {
+void toggle_sol_eol(buffer * const b) {
 
 	if (b->cur_pos == 0) move_to_eol(b);
 	else move_to_sol(b);
@@ -870,52 +886,61 @@ void toggle_sol_eol(buffer *b) {
 
 
 
-/* This function searches for the start of the next or previous word, depending
-on the value of dir. */
+/* Searches for the start of the next or previous word, depending on the value
+	of dir. */
 
-int search_word(buffer *b, int dir) {
+int search_word(buffer * const b, const int dir) {
 
 	line_desc *ld;
-   int y, pos, word_started = FALSE, space_skipped = FALSE;
+	int c, y, pos = b->cur_pos, word_started = FALSE, space_skipped = FALSE;
 
 	assert(dir == -1 || dir == 1);
 
-   ld = b->cur_line_desc;
-	pos = b->cur_pos+dir;
-   y = b->cur_line;
+	ld = b->cur_line_desc;
 
-   while(y<b->line_num && y>=0) {
-		while(pos < ld->line_len && pos>=0) {
-			if ((ispunct((unsigned char)ld->line[pos]) || isspace((unsigned char)ld->line[pos]))) space_skipped = TRUE;
+	if (pos >= ld->line_len) pos = ld->line_len;
+	else {
+		c = get_char(&ld->line[pos], b->encoding);
+		if (utf8ispunct(c) || utf8isspace(c)) space_skipped = TRUE;
+	}
+
+	pos = (dir > 0 ? next_pos : prev_pos)(ld->line, pos, b->encoding);
+
+	y = b->cur_line;
+
+	while(y < b->num_lines && y >= 0) {
+		while(pos < ld->line_len && pos >= 0) {
+			c = get_char(&ld->line[pos], b->encoding);
+			if (utf8ispunct(c) || utf8isspace(c)) space_skipped = TRUE;
 			else word_started = TRUE;
 
-			if (dir>0) {
-				if (space_skipped && !(ispunct((unsigned char)ld->line[pos]) || isspace((unsigned char)ld->line[pos]))) {
+			if (dir > 0) {
+				if (space_skipped && !(utf8ispunct(c) || utf8isspace(c))) {
 					goto_line(b, y);
 					goto_pos(b, pos);
-					return(OK);
+					return OK;
 				}
 			}
 			else {
 				if (word_started) {
-					if (ispunct((unsigned char)ld->line[pos]) || isspace((unsigned char)ld->line[pos])) {
+					if (utf8ispunct(c) || utf8isspace(c)) {
 						goto_line(b, y);
-						goto_pos(b, pos+1);
-						return(OK);
+						goto_pos(b, pos + 1);
+						return OK;
 					}
 					else if (pos == 0) {
 						goto_line(b, y);
 						goto_pos(b, 0);
-						return(0);
+						return 0;
 					}
 				}
 			}
-			pos+=dir;
+			pos = (dir > 0 ? next_pos : prev_pos)(ld->line, pos, b->encoding);
 		}
 
 		space_skipped = TRUE;
 
-		if (dir>0) {
+		if (dir > 0) {
          ld = (line_desc *)ld->ld_node.next;
          y++;
 			pos = 0;
@@ -923,50 +948,51 @@ int search_word(buffer *b, int dir) {
       else {
          ld = (line_desc *)ld->ld_node.prev;
          y--;
-			if (ld->ld_node.prev) pos = ld->line_len-1;
+			if (ld->ld_node.prev) pos = prev_pos(ld->line, ld->line_len, b->encoding);
       }
 	}
-	return(ERROR);
+	return ERROR;
 }
 
 
 
-/* This function moves to the character after the end of the current word. It
-doesn't move at all on spaces and punctuation. */
+/* Moves to the character after the end of the current word. It doesn't move at
+	all on US-ASCII spaces and punctuation. */
 
-void move_to_eow(buffer *b) {
+void move_to_eow(buffer * const b) {
 
 	line_desc *ld = b->cur_line_desc;
-   int pos = b->cur_pos;
+   int pos = b->cur_pos, c;
 
-	if (pos >= ld->line_len  || ispunct((unsigned char)ld->line[pos]) || isspace((unsigned char)ld->line[pos])) return;
+	if (pos >= ld->line_len || utf8ispunct(c = get_char(&ld->line[pos], b->encoding)) || utf8isspace(c)) return;
 
 	while(pos < ld->line_len) {
-		if (ispunct((unsigned char)ld->line[pos]) || isspace((unsigned char)ld->line[pos])) break;
-		pos++;
+		c = get_char(&ld->line[pos], b->encoding);
+		if (utf8ispunct(c) || utf8isspace(c)) break;
+		pos += b->encoding == ENC_UTF8 ? utf8len(ld->line[pos]) : 1;
 	}
 
 	goto_pos(b, pos);
 }
 
 
-/* This function implements Brief's "incrementale move to the end": if we are
-in the middle of a line, we mode to the end of line; otherwise, if we are in
-the middle of a page, we move to the end of the page; otherwise, if we are in
-the middle of a file we move to the end of file. */
+/* Implements Brief's "incrementale move to the end": if we are in the middle
+	of a line, we mode to the end of line; otherwise, if we are in the middle of
+	a page, we move to the end of the page; otherwise, if we are in the middle
+	of a file we move to the end of file. */
 
-void move_inc_down(buffer *b) {
+void move_inc_down(buffer * const b) {
 
 	if (b->cur_pos == b->cur_line_desc->line_len) {
-		if (b->cur_y == ne_lines-2) move_to_bof(b);
+		if (b->cur_y == ne_lines - 2) move_to_bof(b);
 		else next_page(b);
 	}
-	else move_to_eol(b);
+	move_to_eol(b);
 }
 
 /* Same as above, towards the top. */
 
-void move_inc_up(buffer *b) {
+void move_inc_up(buffer * const b) {
 
 	if (b->cur_pos == 0) {
 		if (b->cur_y == 0) move_to_sof(b);
