@@ -1,7 +1,7 @@
 /* main(), global initialization and global buffer functions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2001 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2004 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -36,7 +36,7 @@ message disappears as soon as any key is typed. */
 
 char *NO_WARRANTY_msg[] = {	PROGRAM_NAME " " VERSION ".",
 										"Copyright (C) 1993-1998 Sebastiano Vigna",
-										"Copyright (C) 1999-2001 Todd M. Lewis and Sebastiano Vigna",
+										"Copyright (C) 1999-2004 Todd M. Lewis and Sebastiano Vigna",
 										"",
 										"This program is free software; you can redistribute it and/or modify",
 										"it under the terms of the GNU General Public License as published by",
@@ -48,14 +48,31 @@ char *NO_WARRANTY_msg[] = {	PROGRAM_NAME " " VERSION ".",
 										"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the",
 										"GNU General Public License for more details.",
 										"",
-										"Press F1 or Escape to see the menus. The shortcuts prefixed by ^",
-										"are activated by the Control key; the shortcuts prefixed by [ are",
-										"activated by Control+Meta or just Meta, depending on your terminal",
+										"Press F1, Escape or Escape-Escape to see the menus. The shortcuts",
+										"prefixed by ^ are activated by the Control key; the shortcuts prefixed",
+										"by [ are activated by Control+Meta or just Meta, depending on your terminal",
 										"emulator. Alternatively, just press Escape followed by a letter.",
 										NULL
 									};
 
 
+char ARG_HELP[] = ABOUT_MSG "\n"
+						"Usage: ne [options] [files]\n"
+						"--help        prints this message.\n"
+						"--utf8        use UTF-8 I/O.\n"
+						"--no-utf8     do not use UTF-8 I/O.\n"
+						"--ansi        use built-in ANSI control sequences.\n"
+						"--no-ansi     do not use built-in ANSI control sequences.\n"
+						"--            next token is a filename.\n"
+						"--no-config   do not read configuration files.\n"
+						"--keys FILE   use this file for keyboard configuration.\n"
+						"--menus FILE  use this file for menu configuration.\n"
+						"--macro FILE  exec this macro after start.\n";
+
+/* The regular expression used to parse the locale. */
+
+#define LOCALE_REGEX "\\(UTF-?8\\)\\|\\(ISO-?8859-?\\)\\(1?[0-9]\\)"
+
 /* These lists contains the existing buffers, clips and macros. cur_buffer
 denotes the currently displayed buffer. */
 
@@ -81,7 +98,7 @@ buffer *new_buffer(void) {
 		cur_buffer = b;
 	}
 
-	return(b);
+	return b;
 }
 
 int delete_buffer(void) {
@@ -93,15 +110,15 @@ int delete_buffer(void) {
 
 	if (pb->b_node.prev) {
 		cur_buffer = pb;
-      return(TRUE);
+      return TRUE;
 	}
 
 	if (nb->b_node.next) {
 		cur_buffer = nb;
-		return(TRUE);
+		return TRUE;
 	}
 
-	return(FALSE);
+	return FALSE;
 }
 
 
@@ -116,46 +133,72 @@ int main(int argc, char **argv) {
 	int i, c, no_config = FALSE, displaying_info = FALSE;
 	char *macro_name = NULL, *key_bindings_name = NULL, *menu_conf_name = NULL;
 	clip_desc *cd;
-	char *skiplist;
+	char *skiplist, *locale;
 
 	if ( !(skiplist = calloc( argc, 1 )))  /* We need this many flags. */
 	  exit(1);                             /* This would be bad.       */
 
-	setlocale(LC_ALL, "");
+	if ( locale = setlocale(LC_ALL, "") ) {
+		struct re_pattern_buffer re_pb;
+		struct re_registers re_reg;
+		memset(&re_pb, 0, sizeof re_pb);
+		memset(&re_reg, 0, sizeof re_reg);
 
-	for(i=1; i<argc; i++) {
+		re_pb.translate = (char *)up_case;
+		re_compile_pattern(LOCALE_REGEX, strlen(LOCALE_REGEX), &re_pb);
+		if (re_search(&re_pb, locale, strlen(locale), 0, strlen(locale), &re_reg) >= 0) {
+			if (re_reg.start[1] >= 0) io_utf8 = TRUE;
+		}
+		free(re_reg.start);
+		free(re_reg.end);
+	}
+
+	for(i = 1; i < argc; i++) {
 
 		/* Special arguments start with two dashes. If we find one, we
 		cancel its entry in argv[], so that it will be skipped when opening
 		the specified files. */
 
 		if (argv[i][0] == '-' && argv[i][1] == '-') {
-			if (!strcmp(&argv[i][2], "noconfig" "\0" VERSION_STRING)) {
+			if (!argv[i][2]) skiplist[i++] = 1; /* You can use "--" to force the next token to be a filename */
+			else if (!strcmp(&argv[i][2], "help" "\0" VERSION_STRING)) {
+				puts(ARG_HELP);
+				exit(0);
+			}
+			else if (!strcmp(&argv[i][2], "noconfig") || !strcmp(&argv[i][2], "no-config")) {
 				no_config = TRUE;
 				skiplist[i] = 1; /* argv[i] = NULL; */
 			}
-			if (!strcmp(&argv[i][2], "noansi")) {
+			else if (!strcmp(&argv[i][2], "noansi") || !strcmp(&argv[i][2], "no-ansi")) {
 				ansi = FALSE;
 				skiplist[i] = 1; /* argv[i] = NULL; */
 			}
-			if (!strcmp(&argv[i][2], "ansi")) {
+			else if (!strcmp(&argv[i][2], "ansi")) {
 				ansi = TRUE;
 				skiplist[i] = 1; /* argv[i] = NULL; */
 			}
+			else if (!strcmp(&argv[i][2], "utf8")) {
+				io_utf8 = TRUE;
+				skiplist[i] = 1; /* argv[i] = NULL; */
+			}
+			else if (!strcmp(&argv[i][2], "no-utf8")) {
+				io_utf8 = FALSE;
+				skiplist[i] = 1; /* argv[i] = NULL; */
+			}
 			else if (!strcmp(&argv[i][2], "macro")) {
-				if (i<argc-1) {
+				if (i < argc-1) {
 					macro_name = argv[i+1];
 					skiplist[i] = skiplist[i+1] = 1; /* argv[i] = argv[i+1] = NULL; */
 				}
 			}
 			else if (!strcmp(&argv[i][2], "keys")) {
-				if (i<argc-1) {
+				if (i < argc-1) {
 					key_bindings_name = argv[i+1];
 					skiplist[i] = skiplist[i+1] = 1; /* argv[i] = argv[i+1] = NULL; */
 				}
 			}
 			else if (!strcmp(&argv[i][2], "menus")) {
-				if (i<argc-1) {
+				if (i < argc-1) {
 					menu_conf_name = argv[i+1];
 					skiplist[i] = skiplist[i+1] = 1; /* argv[i] = argv[i+1] = NULL; */
 				}
@@ -234,7 +277,7 @@ int main(int argc, char **argv) {
 
 		stop = FALSE;
 
-		for(i=1; i<argc && !stop; i++) {
+		for(i = 1; i < argc && !stop; i++) {
 			if (argv[i] && !skiplist[i] ) {
 				if (!first_file) do_action(cur_buffer, NEWDOC_A, -1, NULL);
 				else first_file = FALSE;
@@ -264,9 +307,9 @@ int main(int argc, char **argv) {
 
 		displaying_info = TRUE;
 
-		for(i=0; NO_WARRANTY_msg[i]; i++) {
+		for(i = 0; NO_WARRANTY_msg[i]; i++) {
 			move_cursor(i, 0);
-			output_string(NO_WARRANTY_msg[i]);
+			output_string(NO_WARRANTY_msg[i], FALSE);
 		}
 	}
 
@@ -281,7 +324,8 @@ int main(int argc, char **argv) {
 
 		move_cursor(cur_buffer->cur_y, cur_buffer->cur_x);
 
-		ic = char_class[c = get_key_code()];
+		c = get_key_code();
+		ic = CHAR_CLASS(c);
 
 		if ( window_changed_size ) {
 			print_error(do_action(cur_buffer, REFRESH_A, 0, NULL));
@@ -289,10 +333,9 @@ int main(int argc, char **argv) {
 		}
 
 		if (displaying_info) {
-
 			displaying_info = FALSE;
 
-			for(i=0; NO_WARRANTY_msg[i]; i++) {
+			for(i = 0; NO_WARRANTY_msg[i]; i++) {
 				move_cursor(i, 0);
 				clear_to_eol();
 			}
@@ -301,21 +344,26 @@ int main(int argc, char **argv) {
 		}
 
 		switch(ic) {
-			case ALPHA:
-			case TAB:
-				print_error(do_action(cur_buffer, INSERTCHAR_A, c, NULL));
-				break;
+		case INVALID:
+			print_error(INVALID_CHARACTER);
+			break;
+			
+		case ALPHA:
+		case TAB:
+			print_error(do_action(cur_buffer, INSERTCHAR_A, c, NULL));
+			break;
+			
+		case RETURN:
+			print_error(do_action(cur_buffer, INSERTLINE_A, -1, NULL));
+			break;
 
-			case RETURN:
-				print_error(do_action(cur_buffer, INSERTLINE_A, -1, NULL));
-				break;
-
-			case COMMAND:
-				print_error(execute_command_line(cur_buffer, key_binding[c]));
-				break;
-
-			default:
-				break;
+		case COMMAND:
+			if (c < 0) c = -c - 1;
+			if (key_binding[c]) print_error(execute_command_line(cur_buffer, key_binding[c]));
+			break;
+			
+		default:
+			break;
 		}
 	}
 }
