@@ -1,7 +1,7 @@
 /* Main typdefs and defines.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2001 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2004 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -21,7 +21,6 @@
 	02111-1307, USA.  */
 
 
-
 #ifndef TERMCAP
 #include <curses.h>
 #include <term.h>
@@ -31,10 +30,12 @@
 #endif
 
 #include "termchar.h"
+#include "utf8.h"
 
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 #include <fcntl.h>
 #include <ctype.h>
 #include <sys/types.h>
@@ -71,9 +72,21 @@ alphabetic, command-activating, and ignorable character. The class[] vector
 (indexed by the extended character code) gives back the input class. */
 
 typedef enum {
-	ALPHA, COMMAND, RETURN, TAB, IGNORE, INPUT_CLASS_COUNT
+	 ALPHA, COMMAND, RETURN, TAB, IGNORE, INVALID, INPUT_CLASS_COUNT
 } input_class;
 
+/* A buffer or clip at any given time may be marked as ASCII, UTF-8 or
+8-bit. This means, respectively, that it contains just bytes below 0x80, an
+UTF-8-encoded byte sequence or an arbitrary byte sequence.
+
+The attribution is lazy, but stable. This means that a buffer starts as ASCII,
+and becomes UTF-8 or 8-bit as soon as some insertion makes it necessary. This
+is useful to delay the encoding choice as much as possible. */
+
+typedef enum {
+	ENC_ASCII = 0, ENC_UTF8, ENC_8_BIT
+} encoding_type;
+
 
 /* The number of bookmarks per buffer. */
 
@@ -87,10 +100,20 @@ typedef enum {
 
 #define UNNAMED_NAME				"<unnamed>"
 
-/* The number of extended key codes. */
+/* The number of key codes, including ISO-8859-1 plus 256 extra codes. */
 
 #define NUM_KEYS					(256+256)
 
+/* A key code that has class INVALID. */
+
+#define INVALID_CHAR				INT_MIN
+
+/* The mask for commands. */
+
+#define COMMAND_MASK				0x80000000
+
+#define CHAR_CLASS(c) ( ((c)<0) ? (((-c - 1) == INVALID_CHAR) ? INVALID : COMMAND) : ((c)>0xFF) ? ALPHA : char_class[c] )
+
 /* This number is used throughout ne when an integer has to be expanded in
 characters. It is expected that sprintf("%d")ing an integer will produce a
 string shorter than the following number. PORTABILITY PROBLEM: improbable,
@@ -131,13 +154,14 @@ correspondingly. */
 
 typedef struct {
 	int size, len;
-	char *stream;
+	encoding_type encoding;
+	unsigned char *stream;
 } char_stream;
 
 
 #define assert_char_stream(cs) {if ((cs)) {\
 	assert((cs)->len<=(cs)->size);\
-	assert((cs)->len>=0);\
+	assert((cs)->len >= 0);\
 	assert(((cs)->size == 0) == ((cs)->stream == NULL));\
 }}
 
@@ -152,14 +176,16 @@ is NULL iff this node is free for use. */
 
 typedef struct {
 	node ld_node;
-	char *line;
+	unsigned char *line;
 	int line_len;
 } line_desc;
 
-#define assert_line_desc(ld) {if ((ld)) {\
+#define assert_line_desc(ld, encoding) {if ((ld)) {		\
 	assert((ld)->line_len >= 0);\
 	assert(((ld)->line == NULL) == ((ld)->line_len == 0));\
-	assert(((ld)->line_len == 0) || ((ld)->line[0] != 0 && (ld)->line[(ld)->line_len-1] != 0));\
+	assert(((ld)->line_len == 0) || ((ld)->line[0] != 0 && (ld)->line[(ld)->line_len - 1] != 0));\
+	assert(((ld)->line_len == 0) || ((ld)->line[0] != 0 && (ld)->line[(ld)->line_len - 1] != 0)); \
+	if (encoding == ENC_UTF8) { int i = 0; while(i < (ld)->line_len) { assert(utf8len((ld)->line[i]) > 0); i = next_pos((ld)->line, i, encoding);} } \
 }}
 
 
@@ -197,17 +223,17 @@ the source buffer.c for some elaboration on the subject. */
 typedef struct {
 	node cp_node;
 	int  size, first_used, last_used;
-	char *pool;
+	unsigned char *pool;
 } char_pool;
 
 #define assert_char_pool(cp) {if ((cp)) {\
 	assert((cp)->first_used<=(cp)->first_used);\
 	assert((cp)->pool[(cp)->first_used] != 0);\
 	assert((cp)->first_used >= 0);\
-	assert((cp)->first_used == 0 || (cp)->pool[(cp)->first_used-1] == 0);\
+	assert((cp)->first_used == 0 || (cp)->pool[(cp)->first_used - 1] == 0);\
 	assert((cp)->pool[(cp)->last_used] != 0);\
 	assert((cp)->last_used >= 0);\
-	assert((cp)->last_used == (cp)->size-1 || (cp)->pool[(cp)->last_used+1] == 0);\
+	assert((cp)->last_used == (cp)->size - 1 || (cp)->pool[(cp)->last_used + 1] == 0);\
 	assert((cp)->pool != NULL);\
 	assert((cp)->size != 0);\
 }}
@@ -229,7 +255,7 @@ typedef struct struct_macro_desc {
 
 
 /* This structure defines a clip. Clip are numbered from 0 onwards, and
-contain a stream of characters. */
+contain a stream of characters. The stream may be optionally marked as UTF-8. */
 
 typedef struct {
 	node cd_node;
@@ -238,7 +264,7 @@ typedef struct {
 } clip_desc;
 
 #define assert_clip_desc(cd) {if ((cd)) {\
-	assert((cd)->n>=0);\
+	assert((cd)->n >= 0);\
 	assert_char_stream((cd)->cs);\
 }}
 
@@ -261,13 +287,14 @@ typedef struct {
 
 
 /* This structure defines an undo buffer. steps points to an array of
-steps_size undo_steps, used up to cur_step. last_step represent the undo
-step which is the next to be redone in case some undo had place. Note that
-the characters stored in streams, which are used when executing an insertion
-undo step, are not directly pointed to by the undo step. The correct position
-is calculated incrementally, and kept in cur_stream and last_stream.
-Finally, redo contains the stream of characters necessary to perform the
-redo steps. */
+steps_size undo_steps, used up to cur_step. last_step represent the undo step
+which is the next to be redone in case some undo had place. Note that the
+characters stored in streams, which are used when executing an insertion undo
+step, are not directly pointed to by the undo step. The correct position is
+calculated incrementally, and kept in cur_stream and last_stream.  Redo
+contains the stream of characters necessary to perform the redo
+steps. last_save_step is the step (if any) corresponding to the last successful
+buffer save operation. */
 
 typedef struct {
 	undo_step *steps;
@@ -279,6 +306,7 @@ typedef struct {
 	int cur_stream;
 	int last_step;
 	int last_stream;
+	int last_save_step;
 } undo_buffer;
 
 #define assert_undo_buffer(ub) {if ((ub)) {\
@@ -299,7 +327,7 @@ typedef struct {
 	int cur_clip;
 	int tab_size;
 	int right_margin;
-  unsigned int
+	unsigned int
 		free_form:1,       /* Editing is free form (cursor can be anywhere) */
 		status_bar:1,      /* The status bar is displayed */
 		hex_code:1,        /* Show hexadecimal code under the cursor */
@@ -315,11 +343,12 @@ typedef struct {
 		read_only:1,       /* Read-only mode */
 		search_back:1,     /* Last search was backwards */
 		case_search:1,     /* Look at case matching in searches */
-		binary:1;          /* Load and save in binary mode */
+		binary:1,          /* Load and save in binary mode */
+		utf8auto:1;        /* Try to detect automatically UTF-8 */
   } options_t;
 
 #define assert_options(o) {if ((o)) {\
-	assert((o)->tab_size>0);\
+	assert((o)->tab_size > 0);\
 	assert_undo_buffer(&(b)->undo);\
 }}
 
@@ -352,10 +381,11 @@ typedef struct {
 	int wanted_y;             /* desired y position modulo top or bottom of buffer. Valid if y_wanted is TRUE. */
 	int wanted_cur_y;         /* desired cur_y, valid if y_wanted is TRUE */
 	int cur_line, cur_pos;    /* position of cursor withing the doc buffer */
-	int line_num;
-	int block_start_line, block_start_pos, block_start_col;
+	int num_lines;
+	int block_start_line, block_start_col;
 	int allocated_chars;
 	int free_chars;
+	encoding_type encoding;
 	undo_buffer undo;
 	struct {
 		int pos;
@@ -365,7 +395,7 @@ typedef struct {
 	unsigned int
 
 		link_undos,									/* Link the undo steps. Multilevel. */
-		buffer_is_modified:1, 					/* Buffer has been modified since last save */
+		is_modified:1, 					/* Buffer has been modified since last save */
 		recording:1,								/* We are recording a macro */
 		marking:1,									/* We are marking a block */
 		x_wanted:1,									/* We're not where we would like to be */
@@ -388,20 +418,21 @@ typedef struct {
 #define assert_buffer(b) {if ((b)) {\
 	assert((b)->line_desc_list.head->next == NULL || (b)->cur_line_desc != NULL);\
 	assert((b)->line_desc_list.head->next == NULL || (b)->top_line_desc != NULL);\
-	assert_line_desc((b)->cur_line_desc);\
-	assert_line_desc((b)->top_line_desc);\
+	assert_line_desc((b)->cur_line_desc, (b)->encoding);									\
+	assert_line_desc((b)->top_line_desc, (b)->encoding);									\
 	assert_char_stream((b)->last_deleted);\
 	assert_char_stream((b)->cur_macro);\
-	assert((b)->win_y+(b)->cur_y<(b)->line_num);\
-	assert((b)->line_num>0);\
-	assert((b)->opt.tab_size>0);\
+	assert((b)->win_y + (b)->cur_y<(b)->num_lines);\
+	assert((b)->num_lines > 0);\
+	assert((b)->opt.tab_size > 0);\
 	assert((b)->free_chars <= (b)->allocated_chars);\
-	assert((b)->cur_line == (b)->win_y+(b)->cur_y);\
+	assert((b)->cur_line == (b)->win_y + (b)->cur_y);\
+	assert((b)->cur_pos >= (b)->cur_line_desc->line_len || b->encoding != ENC_UTF8 || utf8len((b)->cur_line_desc->line[(b)->cur_pos] > 0));\
 	assert_undo_buffer(&(b)->undo);\
 }}
 
 
-extern const char *key_binding[NUM_KEYS];
+extern const char *key_binding[];
 
 extern buffer *cur_buffer;
 
@@ -431,12 +462,12 @@ extern unsigned int window_changed_size;
 /* This vector associates to an extended key code (as returned by
 get_key_code()) its input class. */
 
-extern const input_class char_class[NUM_KEYS];
+extern const input_class char_class[];
 
 /* This vector is a translation table for the regex library which maps
 lower case characters to upper case characters. It is also used elsewhere. */
 
-extern const unsigned char up_case[256];
+extern const unsigned char up_case[];
 
 /* This number defines the macro hash table size. This table can have
 conflicts. */
@@ -449,6 +480,7 @@ conflicts. */
 #include "names.h"
 #include "errors.h"
 #include "protos.h"
+#include "utf8.h"
 #include "debug.h"
 
 
