@@ -1,7 +1,7 @@
 /* Preferences functions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2001 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2004 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -47,27 +47,27 @@ bigger just causes a reallocation. */
 
 
 
-/* This function returns a pointer to the extension of a filename, or NULL
-if there is no extension. Note that filename has to be non NULL. */
+/* Returns a pointer to the extension of a filename, or NULL if there is no
+   extension. Note that filename has to be non NULL. */
 
-const char *extension(const char *filename) {
+const char *extension(const char * const filename) {
 
 	int i;
 
 	assert(filename != NULL);
 
-	for(i=strlen(filename)-2; i>=0; i--)
-		if (filename[i] == '.') return(&filename[i+1]);
+	for(i = strlen(filename) - 2; i >= 0; i--)
+		if (filename[i] == '.') return &filename[i + 1];
 
-	return(NULL);
+	return NULL;
 }
 
 
 
 
-/* This function returns a pointer to the absolute name of ne's prefs
-directory. The name is cached internally, so it needs not to be free()ed. If
-the directory does not exist, it is created. NULL is returned on failure. */
+/* Returns a pointer to the absolute name of ne's prefs directory. The name is
+   cached internally, so it needs not to be free()ed. If the directory does not
+   exist, it is created. NULL is returned on failure. */
 
 
 char *exists_prefs_dir(void) {
@@ -80,49 +80,49 @@ char *exists_prefs_dir(void) {
 
 	/* If we have been already called, we already computed the name. */
 
-	if (prefs_dir) return(prefs_dir);
+	if (prefs_dir) return prefs_dir;
 
 #ifdef _AMIGA
 	/* The Amiga case is very easy, due to the presence of the PROGDIR: variable.
 	Everything can be handled statically. */
 
 	if (stat(PREFS_DIR, &s)) {
-		if (mkdir(PREFS_DIR)) return(NULL);
+		if (mkdir(PREFS_DIR)) return NULL;
 	}
-	else if (!(s.st_mode & _S_IFDIR)) return(NULL);
+	else if (!(s.st_mode & _S_IFDIR)) return NULL;
 
-	return(prefs_dir = PREFS_DIR);
+	return prefs_dir = PREFS_DIR;
 #else
 	/* In the UN*X case, we first get the home directory. Then
 	we allocate space for the directory name. */
 
 	if (!(home_dir = getenv("HOME"))) home_dir = ".";
 
-	if (prefs_dir = malloc(strlen(home_dir)+strlen(PREFS_DIR)+3)) {
+	if (prefs_dir = malloc(strlen(home_dir) + strlen(PREFS_DIR) + 3)) {
 
 		strcat(strcat(strcpy(prefs_dir, home_dir), "/"), PREFS_DIR);
 
 	   if (stat(prefs_dir, &s)) {
    	   if (mkdir(prefs_dir, 0700)) {
 				free(prefs_dir);
-   	   	return(prefs_dir = NULL);
+   	   	return prefs_dir = NULL;
 			}
 		}
 	   else if (!S_ISDIR(s.st_mode)) {
 			free(prefs_dir);
-	   	return(prefs_dir = NULL);
+	   	return prefs_dir = NULL;
 		}
 
-		return(strcat(prefs_dir, "/"));
+		return strcat(prefs_dir, "/");
 	}
-	else return(NULL);
+	else return NULL;
 #endif
 }
 
 
-/* This function returns a pointer to the absolute name of ne's global prefs
-directory. The name is cached internally, so it needs not to be free()ed. If
-the directory does not exist, it is not created. NULL is returned on failure. */
+/* Returns a pointer to the absolute name of ne's global prefs directory. The
+   name is cached internally, so it needs not to be free()ed. If the directory
+   does not exist, it is not created. NULL is returned on failure. */
 
 char *exists_gprefs_dir(void)
   {
@@ -148,24 +148,24 @@ char *exists_gprefs_dir(void)
             if (stat(gprefs_dir, &s))
                 {
                   free(gprefs_dir);
-                  return(gprefs_dir = NULL);
+                  return gprefs_dir = NULL;
                 }
             else if (!(s.st_mode & _S_IFDIR))
                 {
                   free(gprefs_dir);
-                  return(gprefs_dir = NULL);
+                  return gprefs_dir = NULL;
                 }
             return strcat(gprefs_dir,"/");
           #else
             if (stat(gprefs_dir, &s))
                 {
                   free(gprefs_dir);
-                  return(gprefs_dir = NULL);
+                  return gprefs_dir = NULL;
                 }
             else if (!S_ISDIR(s.st_mode))
                 {
                   free(gprefs_dir);
-                  return(gprefs_dir = NULL);
+                  return gprefs_dir = NULL;
                 }
             return strcat(gprefs_dir, "/");
           #endif
@@ -174,15 +174,15 @@ char *exists_gprefs_dir(void)
   }
 
 
-/* This function saves the preferences of the given buffer onto the given
-file name. If b or name are NULL, ERROR is returned.  */
+/* Saves the preferences of the given buffer onto the given file name. If b or
+   name are NULL, ERROR is returned.  */
 
-int save_prefs(buffer *b, const char *name) {
+int save_prefs(buffer * const b, const char * const name) {
 
 	int error;
 	char_stream *cs;
 
-	if (!b || !name) return(ERROR);
+	if (!b || !name) return ERROR;
 
 	assert_buffer(b);
 
@@ -211,23 +211,23 @@ int save_prefs(buffer *b, const char *name) {
 
 		free_char_stream(cs);
 
-		return(error);
+		return error;
 	}
 
-	return(OUT_OF_MEMORY);
+	return OUT_OF_MEMORY;
 }
 
 
 
-/* This function loads the given preferences file. The file is just executed,
-but with the exec_oxnly_options flag set. If b or name are NULL, ERROR is returned. */
+/* Loads the given preferences file. The file is just executed, but with the
+   exec_oxnly_options flag set. If b or name are NULL, ERROR is returned. */
 
-int load_prefs(buffer *b, const char *name) {
+int load_prefs(buffer * const b, const char * const name) {
 
 	int error = OK;
 	char_stream *cs;
 
-	if (!b || !name) return(ERROR);
+	if (!b || !name) return ERROR;
 
 	assert_buffer(b);
 
@@ -241,54 +241,54 @@ int load_prefs(buffer *b, const char *name) {
 
 	b->exec_only_options = 0;
 
-	return(error);
+	return error;
 }
 
 
 
-/* This function performs an automatic preferences operation, which can be
-loading or saving, depending on the function pointed to by prefs_func. The
-extension given by ext is used in order to locate the appropriate file. If
-ext is NULL, the extension of the buffer filename is used instead. If b is
-NULL, ERROR is returned.  */
+/* Performs an automatic preferences operation, which can be loading or saving,
+   depending on the function pointed to by prefs_func. The extension given by
+   ext is used in order to locate the appropriate file. If ext is NULL, the
+   extension of the buffer filename is used instead. If b is NULL, ERROR is
+   returned.  */
 
-static int do_auto_prefs(buffer *b, const char *ext, int (prefs_func)(buffer *, const char *)) {
+static int do_auto_prefs(buffer *b, const char * ext, int (prefs_func)(buffer *, const char *)) {
 
 	int error = OK;
 	char *auto_name, *prefs_dir;
 
-	if (!b) return(ERROR);
+	if (!b) return ERROR;
 
 	assert_buffer(b);
 
-	if (!ext && (!b->filename || !(ext = extension(b->filename)))) return(HAS_NO_EXTENSION);
+	if (!ext && (!b->filename || !(ext = extension(b->filename)))) return HAS_NO_EXTENSION;
 
 	/* Try global autoprefs -- We always load these before ~/.ne autoprefs.
 	That way the user can override whatever he wants, but anything he
 	doesn't override still gets passed through. */
 	if (*prefs_func == load_prefs && (prefs_dir = exists_gprefs_dir())) {
-		if (auto_name = malloc(strlen(ext)+strlen(prefs_dir)+strlen(PREF_FILE_SUFFIX)+2)) {
+		if (auto_name = malloc(strlen(ext) + strlen(prefs_dir) + strlen(PREF_FILE_SUFFIX) + 2)) {
 			strcat(strcat(strcpy(auto_name, prefs_dir), ext), PREF_FILE_SUFFIX);
 			error = prefs_func(b, auto_name);
 			free(auto_name);
-			/* We don't "return(error);" here because we still haven't loaded
+			/* We don't "return error;" here because we still haven't loaded
 				the user's autoprefs. */
 		}
 	}
 
 	/* Try ~/.ne autoprefs */
 	if (prefs_dir = exists_prefs_dir()) {
-		if (auto_name = malloc(strlen(ext)+strlen(prefs_dir)+strlen(PREF_FILE_SUFFIX)+2)) {
+		if (auto_name = malloc(strlen(ext) + strlen(prefs_dir) + strlen(PREF_FILE_SUFFIX) + 2)) {
 			strcat(strcat(strcpy(auto_name, prefs_dir), ext), PREF_FILE_SUFFIX);
 			error = prefs_func(b, auto_name);
 			free(auto_name);
-			return(error);
+			return error;
 		}
 		else
-			return(OUT_OF_MEMORY);
+			return OUT_OF_MEMORY;
 	}
 
-	else return(CANT_FIND_PREFS_DIR);
+	else return CANT_FIND_PREFS_DIR;
 }
 
 
@@ -296,15 +296,15 @@ static int do_auto_prefs(buffer *b, const char *ext, int (prefs_func)(buffer *,
 load_prefs or save_prefs. */
 
 
-int load_auto_prefs(buffer *b, const char *name) {
+int load_auto_prefs(buffer * const b, const char *name) {
 
-	return(do_auto_prefs(b, name, load_prefs));
+	return do_auto_prefs(b, name, load_prefs);
 }
 
 
-int save_auto_prefs(buffer *b, const char *name) {
+int save_auto_prefs(buffer * const b, const char *name) {
 
-	return(do_auto_prefs(b, name, save_prefs));
+	return do_auto_prefs(b, name, save_prefs);
 }
 
 
@@ -324,7 +324,7 @@ typedef struct
   
 static pref_stack_t pstack = { 0, MAX_PREF_STACK_SIZE };
 
-int push_prefs( buffer *b )
+int push_prefs( buffer * const b )
   {
     char msg[120];
     if ( pstack.pcount >= MAX_PREF_STACK_SIZE )
@@ -340,7 +340,7 @@ int push_prefs( buffer *b )
     return OK;
   }
   
-int pop_prefs( buffer *b )
+int pop_prefs( buffer * const b )
   {
     char msg[120];
     if ( pstack.pcount <= 0 )
