@@ -4,9 +4,6 @@
 	Originally part of GNU Emacs.
 	Vastly edited and modified for use within ne.
 
-	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2001 Todd M. Lewis and Sebastiano Vigna
-
 	This file is part of ne, the nice editor.
 
 	This program is free software; you can redistribute it and/or modify it
@@ -29,8 +26,11 @@
 #include <stdlib.h>
 #include <ctype.h>
 #include <string.h>
+
+#ifndef _AMIGA
 #include <termios.h>
 #include <sys/ioctl.h>
+#endif
 
 #ifndef TERMCAP
 #include <curses.h>
@@ -41,6 +41,7 @@
 
 #include "termchar.h"
 #include "cm.h"
+#include "utf8.h"
 
 
 /* When displaying errors about the terminal database, we try to print the
@@ -162,14 +163,6 @@ the cursor motion strings. */
 struct cm Wcm;
 
 
-/* These are the functions called from cm.c. */
-
-void losecursor (void);
-int Wcm_init (void);
-void cmcostinit (void);
-void cmgoto (int row, int col);
-
-
 #define OUTPUT(a) tputs (a, ne_lines - curY, cmputc)
 #define OUTPUT1(a) tputs (a, 1, cmputc)
 #define OUTPUTL(a, lines) tputs (a, lines, cmputc)
@@ -206,26 +199,119 @@ those operations.  */
 
 int	specified_window;
 
+/* If TRUE, then all I/O is to be performed in UTF-8. */
 
+int	io_utf8;
 
 
 /* PORTABILITY PROBLEM: this function is responsible for filtering nonprintable
-characters. On systems with a wider system character set, it could be
-redefined, for instance, in order to allow characters between 128 and 160 to
-be printed. */
+	characters. On systems with a wider system character set, it could be
+	redefined, for instance, in order to allow characters between 128 and 160 to
+	be printed. Currently, it returns '?' on all control characters (and
+	non-ISO-8859-1 characters, if io_utf8 is false), space on 160, and the
+	obvious capital letter for control characters below 32. */
 
-unsigned char de_control(unsigned char c) {
+int de_control(const int c) {
 	if (c >= 127 && c < 160) return '?';
 	if (c == 160) return ' ';
 	if (c < ' ') return '@'+c;
-	return c;
+	if (c > 0xFF && !io_utf8) return '?';
+
+	/* If io_utf8 is off, we consider all characters in the range of ISO-8859-x
+	encoding schemes as printable. */
+
+	return io_utf8 && wcwidth(c) <= 0 ? '?' : c;
+}
+
+/* Returns the output width of the given character. It is maximised with 1
+ w.r.t. wcwidth(), so its result is equivalent to the width of the character
+ returned by de_control(). */
+
+int output_width(const int c) {
+	const int width = wcwidth(c);
+	return width > 0 ? width : 1;
+}
+
+/* Returns the output width of the given string. If s is NULL, returns len.  If
+the width of the string exceeds maxWidth, modifies len so that it contains the
+longest prefix of s whose width is not greater than maxWidth, and returns the
+corresponding width. */
+
+static int string_output_width(const unsigned char *s, int *len, int maxWidth, int utf8) {
+	if (s == NULL) {
+		if (*len > maxWidth) *len = maxWidth;
+		return *len;
+	}
+	else {
+		int width = 0, char_width, l = *len;
+		if (utf8) {
+			while(l-- != 0) {
+				char_width = output_width(utf8char(s));
+				if (width + char_width > maxWidth) {
+					*len -= l + 1;
+					break;
+				}
+				width += char_width;
+				s += utf8len(*s);
+			}
+		}
+		else while(l-- != 0) {
+			char_width = output_width(*(s++));
+			if (width + char_width > maxWidth) {
+				*len -= l + 1;
+				break;
+			}
+			width += char_width;
+		}
+		return width;
+	}
+}
+
+/* Depending on the value of io_utf8, this function will do a simple putchar(),
+	or a series of putchar() that expand the given character in UTF-8 encoding. */
+
+void out(const int c) {
+	if (io_utf8) {
+		if (c < 0x80) putchar( c ); /* ASCII */
+		else if (c < 0x800) {
+			putchar(0xC0 | (c >> 6));
+			putchar(0x80 | (c >> 0) & 0x3F);
+		}
+		else if (c < 0x10000) {
+			putchar( 0xE0 | (c >> 12));
+			putchar( 0x80 | (c >> 6) & 0x3F );
+			putchar( 0x80 | (c >> 0) & 0x3F );
+		}
+		else if (c < 0x200000) {
+			putchar( 0xF0 | (c >> 18));
+			putchar( 0x80 | (c >> 12) & 0x3F );
+			putchar( 0x80 | (c >> 6) & 0x3F );
+			putchar( 0x80 | (c >> 0) & 0x3F );
+		}
+		else if (c < 0x4000000) {
+			putchar( 0xF8 | (c >> 24));
+			putchar( 0x80 | (c >> 18) & 0x3F );
+			putchar( 0x80 | (c >> 12) & 0x3F );
+			putchar( 0x80 | (c >> 6) & 0x3F );
+			putchar( 0x80 | (c >> 0) & 0x3F );
+		}
+		else {
+			putchar( 0xFC | (c >> 30));
+			putchar( 0x80 | (c >> 24) & 0x3F );
+			putchar( 0x80 | (c >> 18) & 0x3F );
+			putchar( 0x80 | (c >> 12) & 0x3F );
+			putchar( 0x80 | (c >> 6) & 0x3F );
+			putchar( 0x80 | (c >> 0) & 0x3F );
+		}
+	}
+	else putchar(c);
 }
 
 
 
 
-/* These functions ring a bell or flash the screen. If the service
-is not available, the other one is tried. */
+/* Rings a bell or flashes the screen. If the service is not available, the
+	other one is tried. */
 
 
 void ring_bell(void) {
@@ -239,11 +325,11 @@ void do_flash(void) {
 
 
 
-/* This function sets correctly the scroll region (first line is line 0). This
-function assumes scroll_region_ok == TRUE.  The cursor position is lost, as
-from the terminfo specs. */
+/* Sets correctly the scroll region (first line is line 0). This function
+	assumes scroll_region_ok == TRUE.  The cursor position is lost, as from the
+	terminfo specs. */
 
-static void set_scroll_region (int start, int stop) {
+static void set_scroll_region (const int start, const int stop) {
 
 	char *buf;
 
@@ -251,12 +337,8 @@ static void set_scroll_region (int start, int stop) {
 
 	/* Both control string have line range 0 to lines-1 */
 
-	if (ne_change_scroll_region) {
-		buf = tparm (ne_change_scroll_region, start, stop);
-	}
-	else {
-		buf = tparm (ne_set_window, start, stop, 0, ne_columns - 1);
-	}
+	if (ne_change_scroll_region) buf = tparm (ne_change_scroll_region, start, stop);
+	else buf = tparm (ne_set_window, start, stop, 0, ne_columns - 1);
 
 	OUTPUT1(buf);
 	losecursor();
@@ -264,15 +346,13 @@ static void set_scroll_region (int start, int stop) {
 
 
 static void turn_on_insert (void) {
-	if (!insert_mode)
-		OUTPUT1(ne_enter_insert_mode);
+	if (!insert_mode) OUTPUT1(ne_enter_insert_mode);
 	insert_mode = TRUE;
 }
 
 
 static void turn_off_insert (void) {
-	if (insert_mode)
-		OUTPUT1(ne_exit_insert_mode);
+	if (insert_mode) OUTPUT1(ne_exit_insert_mode);
 	insert_mode = FALSE;
 }
 
@@ -282,8 +362,7 @@ but do nothing on terminals with a magic cookie (or without standout).  */
 
 static void turn_off_highlight (void) {
 	if (standout_ok) {
-		if (standout_mode)
-			OUTPUT1(ne_exit_standout_mode);
+		if (standout_mode) OUTPUT1(ne_exit_standout_mode);
 		standout_mode = FALSE;
 	}
 }
@@ -291,14 +370,13 @@ static void turn_off_highlight (void) {
 
 static void turn_on_highlight (void) {
 	if (standout_ok) {
-		if (!standout_mode)
-			OUTPUT1(ne_enter_standout_mode);
+		if (!standout_mode) OUTPUT1(ne_enter_standout_mode);
 		standout_mode = TRUE;
 	}
 }
 
 
-/* This function prepare the terminal for interactive I/O. It
+/* Prepares the terminal for interactive I/O. It
 initializes the terminal, prepares the cursor address mode, and
 activates the keypad and the meta key. */
 
@@ -325,7 +403,7 @@ void set_terminal_modes(void) {
 }
 
 
-/* This function puts again the terminal in its normal state. */
+/* Puts again the terminal in its normal state. */
 
 void reset_terminal_modes (void) {
 
@@ -337,10 +415,10 @@ void reset_terminal_modes (void) {
 }
 
 
-/* This function sets the variable specified_window. Following
-line insert/delete operations will be limited to lines 0 to (size-1). */
+/* Sets the variable specified_window. Following line insert/delete operations
+	will be limited to lines 0 to (size-1). */
 
-void set_terminal_window(int size) {
+void set_terminal_window(const int size) {
 
 	specified_window = size ? size : ne_lines;
 
@@ -377,7 +455,6 @@ void cursor_off (void) {
 /* Set standout mode to the mode specified for the text to be output.  */
 
 static void highlight_if_desired (void) {
-
 	if (standout_requested) turn_on_highlight ();
 	else turn_off_highlight ();
 }
@@ -385,39 +462,29 @@ static void highlight_if_desired (void) {
 
 /* Move to absolute position, specified origin 0 */
 
-void move_cursor (int row, int col) {
-
+void move_cursor (const int row, const int col) {
 	if (curY == row && curX == col) return;
-
 	if (!ne_move_standout_mode) turn_off_highlight ();
-
 	if (!ne_move_insert_mode) turn_off_insert ();
-
 	cmgoto (row, col);
 }
 
 
-/* This function clears from the cursor position to the end of line. It assumes
-that the line is already clear starting at column first_unused_hpos. Note that
-the cursor may be moved, on terminals lacking a `ce' string.  */
+/* Clears from the cursor position to the end of line. It assumes that the line
+	is already clear starting at column first_unused_hpos. Note that the cursor
+	may be moved, on terminals lacking a `ce' string.  */
 
-void clear_end_of_line(int first_unused_hpos) {
+void clear_end_of_line(const int first_unused_hpos) {
 
 	int i;
 
 	if (curX >= first_unused_hpos) return;
 
-	if (ne_clr_eol) {
-		OUTPUT1 (ne_clr_eol);
-	}
+	if (ne_clr_eol) OUTPUT1 (ne_clr_eol);
 	else {
-
 		/* We have to do it the hard way. */
-
 		turn_off_insert ();
-
 		for (i = curX; i < first_unused_hpos; i++) putchar (' ');
-
 		cmplus (first_unused_hpos - curX);
 	}
 }
@@ -431,16 +498,14 @@ void clear_to_eol(void) {
 }
 
 
-/* This function clears from the cursor position to the end of screen */
+/* Clears from the cursor position to the end of screen */
 
 void clear_to_end (void) {
 
-	int i;
-
-	if (ne_clr_eos) {
-		OUTPUT(ne_clr_eos);
-	}
+	if (ne_clr_eos) OUTPUT(ne_clr_eos);
 	else {
+		int i;
+
 		for (i = curY; i < ne_lines; i++) {
 			move_cursor (i, 0);
 			clear_to_eol();
@@ -449,7 +514,7 @@ void clear_to_end (void) {
 }
 
 
-/* This function clears the entire screen */
+/* Clears the entire screen */
 
 void clear_entire_screen (void) {
 
@@ -464,30 +529,32 @@ void clear_entire_screen (void) {
 }
 
 
-/* This function outputs len characters pointed at by string. The characters
-will be truncated to the end of the current line. Passing a NULL for string
-results in outputting spaces. A len of 0 causes no action. */
+/* Outputs len characters pointed at by string. The characters will be
+	truncated to the end of the current line. Passing a NULL for string results
+	in outputting spaces. A len of 0 causes no action. If utf8 is TRUE, the
+	string is UTF-8 encoded. */
 
-void output_chars (const char *string, int len) {
+void output_chars (const unsigned char *string, const int raw_len, const int utf8) {
 
-	const char *p;
-	char *buf;
-	int	c;
-	const char *first_check;
+	const unsigned char *p;
+	unsigned char *buf;
+	int	c, len;
+	const unsigned char *first_check;
 
-	if (len == 0) return;
+	if (raw_len == 0) return;
 
 	highlight_if_desired ();
 	turn_off_insert ();
 
-	if (curX + len > ne_columns) len = ne_columns - curX;
+	/* If the string is UTF-8 encoded, compute its real length. */
+	if (utf8 && string != NULL) len = utf8strlen(string, raw_len);
+	else len = raw_len;
 
-	/* Don't dare write in last column of bottom line, if AutoWrap,
-	since that would scroll the whole screen on some terminals.  */
+	/* If the width of the string exceeds the remaining columns, we reduce
+		len. Moreovero, we don't dare write in last column of bottom line, if
+		AutoWrap, since that would scroll the whole screen on some terminals. */
 
-	if (AutoWrap && curY + 1 == ne_lines && curX + len == ne_columns) len--;
-
-	cmplus (len);
+	cmplus(string_output_width(string, &len, ne_columns - curX - (AutoWrap && curY == ne_lines - 1), utf8));
 
 	if (string == NULL) {
 		if (len > RPov) {
@@ -504,13 +571,20 @@ void output_chars (const char *string, int len) {
 
 	if (RPov > len && !ne_transparent_underline && !ne_tilde_glitch) {
 		while(--len >= 0) {
-			c = *string++;
-			putchar(de_control(c));
+			if (utf8) {
+				c = utf8char(string);
+				string += utf8len(*string);
+				out(de_control(c));
+			}
+			else {
+				c = *string++;
+				out(de_control(c));
+			}
 		}
 	} else
 		while (--len >= 0) {
-			c = *string;
-			if (RPov + 1 < len && string >= first_check) {
+			c = utf8 ? *string : utf8char(string);
+			if (!io_utf8 && RPov + 1 < len && string >= first_check) {
 				int	repeat_count;
 
 				p = string;
@@ -543,49 +617,96 @@ void output_chars (const char *string, int len) {
 			if (ne_tilde_glitch && c == '~')
 				c = '`';
 
-			putchar (de_control(c));
-			string++;
+			out(de_control(c));
+			string += utf8 ? utf8len(*string) : 1;
 		}
 }
 
 
 
+/* Outputs a NULL-terminated string. */
+
+void output_string(const char * const s, const int utf8) {
+	assert(s != NULL);
+	output_chars(s, strlen(s), utf8);
+}
+
+
+/* Outputs a single ISO 10646 character. */
+
+void output_char(const int c, const int utf8) {
+	static unsigned char t[8];
+
+	if (utf8) {
+		memset(t, 0, sizeof t);
+		utf8str(c, t);
+	}
+	else {
+		t[0] = c;
+		t[1] = 0;
+	}
+
+	assert(c != 0);
+
+	output_string(t, utf8);
+}
+
+
+/* Outputs spaces. */
+
+void output_spaces(const int n) {
+   output_chars(NULL, n, FALSE);
+}
+
 /* Same as output_chars(), but inserts instead. */
 
-void insert_chars (const char *start, int len) {
+void insert_chars (const unsigned char *start, const int raw_len, const int utf8) {
 
-	char *buf;
-	int c;
+	const unsigned char *buf;
+	int c, len, width;
 
-	if (len == 0) return;
+	if (raw_len == 0) return;
 
-	highlight_if_desired ();
+	highlight_if_desired();
+
+	/* If the string is non-NULL and UTF-8 encoded, compute its real length. */
+	if (utf8 && start != NULL) len = utf8strlen(start, raw_len);
+	else len = raw_len;
 
 	if (ne_parm_ich) {
+		int i, width = 0;
 
-		buf = tparm (ne_parm_ich, len);
+		if (start != NULL) {
+			if (utf8) for(i = 0; i < raw_len; i += utf8len(start[i])) width += output_width(utf8char(start + i));
+			else for(i = 0; i < raw_len; i++) width += output_width(start[i]);
+		}
+		else width = len;
+
+		buf = tparm (ne_parm_ich, width);
 		OUTPUT1 (buf);
 
-		if (start) output_chars (start, len);
+		if (start) output_chars (start, raw_len, utf8);
 
 		return;
 	}
 
-	if (curX + len > ne_columns) len = ne_columns - curX;
-
-	/* Don't dare write in last column of bottom line, if AutoWrap,
-	since that would scroll the whole screen on some terminals.  */
-
-	if (AutoWrap && curY + 1 == ne_lines && curX + len == ne_columns) len--;
-
 	turn_on_insert ();
-	cmplus (len);
+
+	/* If the width of the string exceeds the remaining columns, we reduce
+		len. Moreovero, we don't dare write in last column of bottom line, if
+		AutoWrap, since that would scroll the whole screen on some terminals. */
+
+	cmplus(string_output_width(start, &len, ne_columns - curX - (AutoWrap && curY == ne_lines - 1), utf8));
 
 	if (!ne_transparent_underline && !ne_tilde_glitch && start
 	     && ne_insert_padding == NULL && ne_insert_character == NULL) {
 		while(--len >= 0) {
-			c = *start++;
-			putchar(de_control(c));
+			if (utf8) {
+				c = utf8char(start);
+				start += utf8len(*start);
+			}
+			else c = *start++;
+			out(de_control(c));
 		}
 	}
 	else
@@ -595,26 +716,49 @@ void insert_chars (const char *start, int len) {
 
 			if (!start) c = ' ';
 			else {
-				c = *start++;
+				if (utf8) {
+					c = utf8char(start);
+					start += utf8len(*start);
+				}
+				else c = *start++;
+				
 				if (ne_tilde_glitch && c == '~') c = '`';
 			}
 
-			putchar (de_control(c));
+			out(de_control(c));
 			OUTPUT1_IF(ne_insert_padding);
 		}
 }
 
 
+/* Inserts a single ISO 10646 character. */
 
-/* This function deletes n characters at the current cursor position. */
+void insert_char(const int c, const int utf8) {
+	static unsigned char t[8];
+
+	if (utf8) {
+		memset(t, 0, sizeof t);
+		utf8str(c, t);
+	}
+	else {
+		t[0] = c;
+		t[1] = 0;
+	}
+
+	assert(c != 0);
+
+	insert_chars(t, strlen(t), utf8);
+}
+
+/* Deletes n characters at the current cursor position. */
 
 void delete_chars (int n) {
 
 	char *buf;
 
-	if (delete_in_insert_mode) {
-		turn_on_insert();
-	}
+	if ( n == 0 ) return;
+
+	if (delete_in_insert_mode) turn_on_insert();
 	else {
 		turn_off_insert();
 		OUTPUT1_IF(ne_enter_delete_mode);
@@ -624,11 +768,9 @@ void delete_chars (int n) {
 		buf = tparm(ne_parm_dch, n);
 		OUTPUT1(buf);
 	}
-	else
-		while(--n>=0) OUTPUT1(ne_delete_character);
+	else while(--n >= 0) OUTPUT1(ne_delete_character);
 
-	if (!delete_in_insert_mode)
-		OUTPUT_IF(ne_exit_delete_mode);
+	if (!delete_in_insert_mode) OUTPUT_IF(ne_exit_delete_mode);
 }
 
 
@@ -636,26 +778,21 @@ void delete_chars (int n) {
 for n lines, given a parametrized and/or a one-line capability
 for that purpose. */
 
-static void do_multi_ins_del(char *multi, char *single, int n) {
+static void do_multi_ins_del(char * const multi, const char * const single, int n) {
 
 	if (multi) {
-
-		char *buf;
-
-		buf = tparm(multi, n);
+		char *const buf = tparm(multi, n);
 		OUTPUT(buf);
 	}
-	else {
-		while(--n>=0) OUTPUT(single);
-	}
+	else while(--n >= 0) OUTPUT(single);
 }
 
 
-/* This function inserts n lines at vertical position vpos. If n is negative,
-it deletes -n lines. specified_window is taken into account. This function
-assumes line_ins_del_ok == TRUE. */
+/* Inserts n lines at vertical position vpos. If n is negative, it deletes -n
+	lines. specified_window is taken into account. This function assumes
+	line_ins_del_ok == TRUE. */
 
-void ins_del_lines (int vpos, int n) {
+void ins_del_lines (const int vpos, const int n) {
 
 	int i = n > 0 ? n : -n;
 
@@ -663,28 +800,23 @@ void ins_del_lines (int vpos, int n) {
 	assert(i != 0);
 	assert(vpos < specified_window);
 
-	if (scroll_region_ok && vpos + i >= specified_window)
-		return;
+	if (scroll_region_ok && vpos + i >= specified_window) return;
 
-	if (!ne_memory_below && vpos + i >= ne_lines)
-		return;
+	if (!ne_memory_below && vpos + i >= ne_lines) return;
 
 	if (scroll_region_ok) {
-		if (specified_window != ne_lines)
-			set_scroll_region(vpos, specified_window-1);
+		if (specified_window != ne_lines) set_scroll_region(vpos, specified_window - 1);
 
 		if (n < 0) {
 			move_cursor(specified_window - 1, 0);
-			while (--i >= 0)
-				OUTPUTL(ne_scroll_forward, specified_window-vpos+1);
+			while (--i >= 0) OUTPUTL(ne_scroll_forward, specified_window - vpos + 1);
 		}
 		else {
 			move_cursor(vpos, 0);
-			while (--i >= 0)
-				OUTPUTL(ne_scroll_reverse, specified_window-vpos+1);
+			while (--i >= 0) OUTPUTL(ne_scroll_reverse, specified_window - vpos + 1);
 		}
 
-		if (specified_window != ne_lines) set_scroll_region(0, ne_lines-1);
+		if (specified_window != ne_lines) set_scroll_region(0, ne_lines - 1);
 	}
 	else {
 		if (n > 0) {
@@ -701,7 +833,7 @@ void ins_del_lines (int vpos, int n) {
 			do_multi_ins_del(ne_parm_delete_line, ne_delete_line, i);
 
 			if (specified_window != ne_lines) {
-				move_cursor(specified_window-i, 0);
+				move_cursor(specified_window - i, 0);
 				do_multi_ins_del(ne_parm_insert_line, ne_insert_line, i);
 			}
 			else if (ne_memory_below) {
@@ -717,20 +849,20 @@ extern int cost;		/* In cm.c */
 extern int evalcost(int);
 
 
-/* This function performs the cursor motion cost setup, and sets the
-variable RPov to the number of characters (with padding) which are
-really output when repeating one character. */
+/* Performs the cursor motion cost setup, and sets the variable RPov to the
+	number of characters (with padding) which are really output when repeating
+	one character. RPov is disable using UTF-8 I/O. */
 
 static void calculate_costs (void) {
 
 	if (ne_repeat_char) {
 
-		char *buf = tparm(ne_repeat_char, ' ', 1);
+		char *const buf = tparm(ne_repeat_char, ' ', 1);
 
 		cost = 0;
 		tputs(buf, 1, evalcost);
 
-		RPov = cost+1;
+		RPov = cost + 1;
 	}
 	else RPov = ne_columns * 2;
 
@@ -739,9 +871,9 @@ static void calculate_costs (void) {
 
 
 
-/* This function gets the window size using TIOCGWINSZ, or does nothing if
-TIOCGWINSZ is not available. It is called by the signal handler for SIGWINCH on
-systems that support it. Return 1 if the window size has changed. */
+/* Gets the window size using TIOCGWINSZ, or does nothing if TIOCGWINSZ is not
+	available. It is called by the signal handler for SIGWINCH on systems that
+	support it. Return 1 if the window size has changed. */
 
 int ttysize(void) {
 #ifdef TIOCGWINSZ
@@ -755,8 +887,8 @@ int ttysize(void) {
 		if ( (ne_lines != size.ws_row) || ( ne_columns != size.ws_col ) ) {
 			ScreenRows = ne_lines    = size.ws_row;
 			ScreenCols = ne_columns  = size.ws_col;
-			set_terminal_window(ne_lines-1);
-			if (scroll_region_ok) set_scroll_region(0, ne_lines-1);
+			set_terminal_window(ne_lines - 1);
+			if (scroll_region_ok) set_scroll_region(0, ne_lines - 1);
 			D(fprintf(stderr,"ttysize: size changed.\n");)
 			return 1;
 		}
